import sys, math, pygame, psyco
from pygame.locals import *

scr_width = 640
scr_height = 480
	
dest_x_res = 1024.0
dest_y_res = 768.0

def init_display():
	return pygame.display.set_mode((scr_width, scr_height))

class Vector:
	def __init__(self, x, y):
		self.x = x
		self.y = y

	def __len__(self):
		return 2

	def __getitem__(self, n):
		if n == 0:
			return self.x
		else:
			return self.y

	def len2(self):
		return self.x * self.x + self.y*self.y

	def len(self):
		return math.sqrt(self.len2())

	def dist2(self, v):
		dx, dy = v.x - self.x, v.y - self.y
		return dx*dx + dy*dy

	def dist(self, v):
		return math.sqrt(self.dist2(v))

	def add(self, v):
		return Vector(self.x + v.x, self.y + v.y)
	
	def sub(self, v):
		return Vector(self.x - v.x, self.y - v.y)

	def reverse(self):
		return Vector(-self.x, -self.y)

	def dot_product(self, v):
		return self.x * v.x + self.y * v.y

	def scale(self, factor):
		return Vector(self.x * factor, self.y * factor)

#	def v2tup(self):
#		return (self.x, self.y)

def t2vec(tuple):
	return Vector(tuple[0], tuple[1])

class Line:
	def __init__(self, normal, pt):
		self.a = normal.x
		self.b = normal.y
		self.c = -self.a*pt.x - self.b*pt.y
		
	def intersect(self, l):
		a, b, c = self.a, self.b, self.c
		d, e, f = l.a, l.b, l.c

		return Vector((e*c - b*f) / (b*d - e*a), (d*c - a*f) / (e*a - b*d))

class Arc:
	def __init__(self, center, radius, start_ang, end_ang, start = Vector(0,0), end = Vector(0,0), dir = Vector(0, 0), rdir = Vector(0, 0)):
		self.cp = center
		self.r = radius
		self.sang = start_ang
		self.eang = end_ang
		self.s = start
		self.e = end
		self.dir = dir
		self.rdir = rdir

	def recalc(self):
		self.s = Vector(self.cp.x + math.cos(self.sang)*self.r, self.cp.y - math.sin(self.sang)*self.r)
		self.e = Vector(self.cp.x + math.cos(self.eang)*self.r, self.cp.y - math.sin(self.eang)*self.r)
		l1 = Line(self.s.sub(self.cp), self.s)
		l2 = Line(self.e.sub(self.cp), self.e)
		mid_pt = l1.intersect(l2)
		
		sg = Segment(self.s, mid_pt)
		self.dir = sg.dir
		
		sg = Segment(mid_pt, self.e)
		self.rdir = sg.dir

	def clone(self):
		a = Arc(self.cp, self.r, self.sang, self.eang, self.s, self.e, self.dir)
		return a

	def reverse(self):
		self.sang, self.eang = self.eang, self.sang
		self.s, self.e = self.e, self.s
		self.dir, self.rdir = self.rdir, self.dir

	def project(self, pt):
		sg = Segment(self.s, self.e)
		if not sg.sees(pt):
			return None

		cp_pt = Segment(self.cp, pt)
		
		dx, dy = cp_pt.e.x - cp_pt.s.x, cp_pt.e.y - cp_pt.s.y
		dr = dx*dx + dy*dy
		D = (cp_pt.s.x - self.cp.x) * (cp_pt.e.y - self.cp.y) + (cp_pt.e.x-self.cp.x) * (cp_pt.s.y - self.cp.y)
		if dy < 0:
			sgndy = -1
		else:
			sgndy = 1

		disc = math.sqrt(self.r*self.r*dr - D*D )

		p1 = Vector( (D*dy + sgndy*dx*disc) / dr + self.cp.x, (-D*dx + math.fabs(dy)*disc) / dr + self.cp.y )
		p2 = Vector( (D*dy - sgndy*dx*disc) / dr + self.cp.x, (-D*dx - math.fabs(dy)*disc) / dr + self.cp.y )

		chord = Segment(self.s, self.e)
		ch_pt = chord.project(pt)

		if ch_pt.dist2(p1) < ch_pt.dist2(p2):
			return p1
		else:
			return p2

	def draw(self, surf, color = (255, 255, 255), ar_length = 5):
		ang_step = ar_length / self.r
		
		ang, end_ang = self.sang, self.eang
		if ang > end_ang:
			ang, end_ang = end_ang, ang
		if end_ang - ang > math.pi:
			ang, end_ang = end_ang, ang + 2*math.pi
		prev_pos = (self.cp.x + self.r * math.cos(ang), self.cp.y - self.r * math.sin(ang))
		while ang<end_ang:
			ang = ang + ang_step
			act_pos = (self.cp.x + self.r * math.cos(ang), self.cp.y - self.r * math.sin(ang))
			pygame.draw.line(surf, color, prev_pos, act_pos)
			prev_pos = act_pos

class Segment:
	def __init__(self, start, end):
		self.s = start
		self.e = end
		self.se = end.sub(start)
		self.es = start.sub(end)
		self.l = Line(Vector(-self.se.y, self.se.x), self.s)
		self.length = self.se.len()
		self.dir =  self.se.scale(1/self.length)
		self.rdir = self.dir.reverse()

	def recalc(self):
		self.se = self.e.sub(self.s)
		self.es = self.s.sub(self.e)
		self.l = Line(Vector(-self.se.y, self.se.x), self.s)
		self.length = self.se.len()
		if self.length > 0:
			self.dir =  self.se.scale(1/self.length)
			self.rdir = self.dir.reverse()
		else:
			self.dir = self.rdir = Vector(0, 0)

	def reverse(self):
		self.s, self.e = self.e, self.s
		self.se, self.es = self.es, self.se
		self.dir, self.rdir = self.rdir, self.dir

	def sees(self, v):
		sv, ev = v.sub(self.s), v.sub(self.e)
		return self.se.dot_product(sv) >= 0 and self.es.dot_product(ev) >= 0

	def project(self, v, crop_to_end = False):
		if self.sees(v):
			l = Line(self.dir, v)
			return self.l.intersect(l)
		elif crop_to_end:
			sv, ev = v.sub(self.s), v.sub(self.e)
			if self.se.dot_product(sv) < 0:
				return self.s
			else:
				return self.e

	def calc_arc(self, end_segment, dist, crop_dist = True):
		if crop_dist:
			dist = min(dist, self.length, end_segment.length)
		
		p1 = self.dir.scale(-dist).add(self.e)
		p2 = end_segment.dir.scale(dist).add(end_segment.s)
		midpt = self.e
		
		l1 = Line(self.dir, p1)
		l2 = Line(end_segment.dir, p2)
		cp = l1.intersect(l2)
		r = cp.dist(p1)
		
		sa = abs(p2.y - cp.y) / r
		if sa > 1:
			sa = 1
	
		if p2.x >= cp.x and p2.y <= cp.y:
			alpha = math.asin(sa)
		elif p2.x <= cp.x and p2.y <= cp.y:
			alpha = math.pi - math.asin(sa)
		elif p2.x <= cp.x and p2.y >= cp.y: 
			alpha = math.pi + math.asin(sa)
		else:
			alpha = 2*math.pi - math.asin(sa)
			
		sb = abs(p1.y - cp.y) / r
		if sb > 1:
			sb = 1
			
		if p1.x >= cp.x and p1.y <= cp.y:
			beta = math.asin(sb)
		elif p1.x <= cp.x and p1.y <= cp.y:
			beta = math.pi - math.asin(sb)
		elif p1.x <= cp.x and p1.y >= cp.y: 
			beta = math.pi + math.asin(sb)
		else:
			beta = 2*math.pi - math.asin(sb)
	
		# Only swap them for drawing!!!
		#if alpha > beta:
		#	alpha, beta = beta, alpha

		#if beta - alpha > math.pi:
		#	alpha, beta = beta, alpha + 2*math.pi

		#return Arc(cp, r, alpha, beta, p1, p2, self.rdir)
		return Arc(cp, r, beta, alpha, p1, p2, self.dir, end_segment.dir)


class RailSegment:
	def __init__(self, segment, arc = None):
		self.sg = segment
		self.arc = arc

class RailLine:
	def __init__(self):
		self.segments = []
		self.start_switch = None
		self.end_switch = None
		self.coal_start_segment = None
		self.coal_end_segment = None
		self.coal_start_pt = None
		self.coal_end_pt = None
		self.coal_start_ang = None
		self.coal_end_ang = None
		self.coal_stack = None
		self.spawn_line = False

	def __len__(self):
		return len(self.segments)

	def reverse(self):
		self.segments.reverse()
		for s in self.segments:
			s.sg.reverse()
		
			if s.arc != None:
				s.arc.reverse()
		
		if self.start_switch != None:
			i = self.start_switch.lines.index([self, Switch.LINE_START])
			self.start_switch.lines[i][1] = Switch.LINE_END
		
		if self.end_switch != None:
			i = self.end_switch.lines.index([self, Switch.LINE_END])
			self.end_switch.lines[i][1] = Switch.LINE_START
		
		self.start_switch, self.end_switch = self.end_switch, self.start_switch
	
	def append(self, segment):
		self.segments.append(segment)
		if len(self.segments) > 1:
			segment.sarc = self.segments[-2].arc

	def insert(self, segment):
		self.segments.insert(0, segment)
		if len(self.segments) > 1:
			self.segments[0].sarc = segment.arc

	def get_coal_positions(self):
		if self.coal_start_segment == None:
			return None

		l = 0

		for act_seg in self.segments:
			if act_seg == self.coal_start_segment:
				if act_seg == self.coal_end_segment:
					if act_seg.arc != None:
						adiff = math.fabs(self.coal_start_ang - act_seg.arc.sang)	
						if adiff > math.pi:
							adiff = math.pi*2 - adiff

						coal_start = l + act_seg.arc.r*adiff
			
						adiff = math.fabs(self.coal_end_ang - self.coal_start_ang)	
						if adiff > math.pi:
							adiff = math.pi*2 - adiff
				
						coal_end = coal_start + act_seg.arc.r*adiff
					else:
						coal_start = l + act_seg.sg.s.dist(self.coal_start_pt)
						coal_end = coal_start + self.coal_end_pt.dist(self.coal_start_pt)
					break

				else:
					if act_seg.arc != None:
						adiff = math.fabs(self.coal_start_ang - act_seg.arc.sang)
						if adiff > math.pi:
							adiff = math.pi*2 - adiff

						coal_start = l + act_seg.arc.r*adiff
			
						adiff = math.fabs(act_seg.arc.eang - act_seg.arc.sang)	
						if adiff > math.pi:
							adiff = math.pi*2 - adiff
				
						l = l + act_seg.arc.r*adiff
					else:
						coal_start = l + act_seg.sg.s.dist(self.coal_start_pt)
						l = l + act_seg.sg.length
			elif act_seg == self.coal_end_segment:
				if act_seg.arc != None:
					adiff = math.fabs(self.coal_end_ang - act_seg.arc.sang)
					if adiff > math.pi:
						adiff = math.pi*2 - adiff

					coal_end = l + act_seg.arc.r*adiff
				else:
					coal_end = l + act_seg.sg.s.dist(self.coal_end_pt)
				break

			else:
				if act_seg.arc != None:
					adiff = math.fabs(act_seg.arc.eang - act_seg.arc.sang)
					if adiff > math.pi:
						adiff = math.pi*2 - adiff
						
					l = l + act_seg.arc.r*adiff
				else:
					l = l + act_seg.sg.length
		
		return (coal_start, coal_end)

class Switch:
	LINE_START = 0
	LINE_END = 1
	def __init__(self, line1 = None, end1 = None, line2 = None, end2 = None):
		if line1 == None:
			self.lines = []
			self.states = []
			return

		self.lines = [ [line1, end1], [line2, end2] ]
		if end1 == self.LINE_START:
			line1.start_switch = self
		else:
			line1.end_switch = self
		
		if end2 == self.LINE_START:
			line2.start_switch = self
		else:
			line2.end_switch = self

		self.states = []

	def add_line(self, line, end):
		self.lines.append([line, end])
		if end == self.LINE_START:
			line.start_switch = self
		else:
			line.end_switch = self

	def remove_line(self, line, end):
		self.lines.remove([line, end])
		if end == self.LINE_START:
			line.start_switch = None
		else:
			line.end_switch = None

	def add_state(self, line1, line2):
		self.states.append([line1, line2])

	def replace_references(self, old_line, new_line, direction):
		n = 0
		for l in self.lines:
			if l[0] is old_line:
				n = n + 1
				l[0] = new_line

		print ("Replaced %d line references" % n)

		n = 0
		for s in self.states:
			if s[0] is old_line:
				n = n +1
				s[0] = new_line
			elif s[1] is old_line:
				n = n +1
				s[1] = new_line
		
		print ("Replaced %d state references" % n)

		if direction == self.LINE_START:
			new_line.start_switch = old_line.start_switch
		else:
			new_line.end_switch = old_line.end_switch

class SnapInfo:
	SEGMENT_START = 0
	SEGMENT_END = 1
	LINE_MID = 2
	ARC_MID = 3

	def __init__(self, line, segment, dist, pt, type, angle = None):
		self.line = line
		self.segment = segment
		self.dist = dist
		self.type = type
		self.pt = pt
		self.angle = angle

def find_snap_point(rail_lines, reference_pt, pt_snap_dist, line_snap_dist, arc_snap_dist, exclude_lines = []):
	min_dist = -1

	# search for points to snap to
	if pt_snap_dist != -1:
		for act_line in rail_lines:
			if act_line in exclude_lines:
				continue
			if len(act_line) > 0:
				for act_segment in act_line.segments:
					act_dist = act_segment.sg.s.dist2(reference_pt)
					if act_dist < pt_snap_dist and (min_dist == -1 or min_dist > act_dist):
						min_dist = act_dist
						min_segment = act_segment
						min_line = act_line
						type = SnapInfo.SEGMENT_START
						pt = act_segment.sg.s

				act_dist = act_line.segments[-1].sg.e.dist2(reference_pt)
				if act_dist < pt_snap_dist and (min_dist == -1 or min_dist > act_dist):
					min_dist = act_dist
					min_segment = act_line.segments[-1]
					min_line = act_line
					type = SnapInfo.SEGMENT_END
					pt = act_line.segments[-1].sg.e
	
		if min_dist != -1:
			return SnapInfo(min_line, min_segment, min_dist, pt, type)
#			return (min_line, min_segment, min_dist)

	# search for lines to snap to
	if line_snap_dist != -1:
		for act_line in rail_lines:
			if act_line in exclude_lines:
				continue
			for act_segment in act_line.segments:
				if act_segment.arc != None:
					continue
				pr_pt = act_segment.sg.project(reference_pt)
				if pr_pt != None:
					act_dist = pr_pt.dist2(reference_pt)
					if act_dist < pt_snap_dist and (min_dist == -1 or min_dist > act_dist):
						min_dist = act_dist
						min_segment = act_segment
						min_line = act_line
						type = SnapInfo.LINE_MID
						pt = pr_pt

		if min_dist != -1:
			return SnapInfo(min_line, min_segment, min_dist, pt, type)

	if arc_snap_dist != -1:
		for act_line in rail_lines:
			if act_line in exclude_lines:
				continue
			for act_segment in act_line.segments:
				if act_segment.arc == None:
					continue
				pr_pt = act_segment.arc.project(reference_pt)
				if pr_pt != None:
					act_dist = pr_pt.dist2(reference_pt)
					if act_dist < arc_snap_dist and (min_dist == -1 or min_dist > act_dist):
						min_dist = act_dist
						min_segment = act_segment
						min_line = act_line
						type = SnapInfo.ARC_MID
						pt = pr_pt

						cp = act_segment.arc.cp

						sa = abs(pt.y - cp.y) / act_segment.arc.r
						if sa > 1:
							sa = 1
					
						if pt.x >= cp.x and pt.y <= cp.y:
							alpha = math.asin(sa)
						elif pt.x <= cp.x and pt.y <= cp.y:
							alpha = math.pi - math.asin(sa)
						elif pt.x <= cp.x and pt.y >= cp.y: 
							alpha = math.pi + math.asin(sa)
						else:
							alpha = 2*math.pi - math.asin(sa)

		if min_dist != -1:
			return SnapInfo(min_line, min_segment, min_dist, pt, type, alpha)

class HelperPoint:
	def __init__(self, pos = Vector(0,0), visible = False, color = (255, 0, 0)):
		self.pos = pos
		self.color = color
		self.visible = visible

	def draw(self, disp):
		if self.visible:
			pygame.draw.circle(disp, self.color, self.pos, 2)

class HelperSnap:
	def __init__(self, snap = None, color = (255, 0, 0)):
		self.snap = snap
		self.color = color
		self.visible = False

	def draw(self, disp):
		if self.visible:
			pygame.draw.circle(disp, self.color, self.snap.pt, 2)

class HelperLine:
	def __init__(self, color = (255, 0, 0)):
		self.s = None
		self.e = None
		self.c = color
		self.visible = False
	
	def show(self, init_start, init_end = None):
		self.s = init_start
		if init_end != None:
			self.e = init_end
		else:
			self.e = init_start

		self.visible = True

	def hide(self):
		self.visible = False

	def draw(self, disp):
		if self.visible:
			pygame.draw.line(disp, self.c, self.s, self.e)

class HelperArc:
	def __init__(self, arc, color = (0, 255, 0)):
		self.a = arc
		self.c = color
		self.enabled = True

	def draw(self, disp):
		if self.enabled:
			#r = self.a.r
			#if r < 1:
			#	r = 1
			#rect = pygame.Rect(self.a.cp.x - r, self.a.cp.y - r, r*2, r*2)
			#pygame.draw.arc(disp, self.c, rect, self.a.sang, self.a.eang)
			self.a.draw(disp, self.c)

class HelperRailLine:
	def __init__(self, rail_line = None, head_color = (0, 255, 0), tail_color = (0, 0, 255), line_color = (251, 242, 100), arc_color = (148, 147, 73)):
		self.rail_line = rail_line
		self.head = head_color
		self.tail = tail_color
		self.line = line_color
		self.arc = arc_color
		self.visible = False

		font = pygame.font.SysFont("Sans", 10)
		self.start_text = font.render("S", True, (255, 255, 255))
		self.end_text = font.render("E", True, (255, 255, 255))

		self.coal_start_text = font.render("CS", True, (200,200,200))
		self.coal_end_text = font.render("CE", True, (200,200,200))

	def draw(self, disp):
		line_color = self.line
		arc_color = self.arc
		coal_color = (200, 200, 200)
		
		if self.visible and len(self.rail_line) > 0:
			for act_segment in self.rail_line.segments:
				if act_segment.arc == None:
					pygame.draw.line(disp, line_color, act_segment.sg.s, act_segment.sg.e, 2)
				else:
					act_segment.arc.draw(disp, arc_color)

				if act_segment is self.rail_line.coal_start_segment:
					if act_segment is self.rail_line.coal_end_segment:
						if act_segment.arc == None:
							pygame.draw.line(disp, coal_color, self.rail_line.coal_start_pt, self.rail_line.coal_end_pt,2)
						else:
							a = act_segment.arc.clone()
							a.sang = self.rail_line.coal_start_ang
							a.eang = self.rail_line.coal_end_ang
							a.draw(disp, coal_color)
					else:
						if act_segment.arc == None:
							pygame.draw.line(disp, coal_color, self.rail_line.coal_start_pt, act_segment.sg.e,2)
						else:
							a = act_segment.arc.clone()
							a.sang = self.rail_line.coal_start_ang
							a.draw(disp, coal_color)
						line_color = arc_color = coal_color
					
				elif act_segment is self.rail_line.coal_end_segment:
					line_color = self.line
					arc_color = self.arc
					if act_segment.arc == None:
						pygame.draw.line(disp, line_color, self.rail_line.coal_end_pt, act_segment.sg.e,2)
					else:
						a = act_segment.arc.clone()
						a.sang = self.rail_line.coal_end_ang
						a.draw(disp, arc_color)

			#pygame.draw.circle(disp, self.head, self.rail_line.segments[0].sg.s, 3)
			#pygame.draw.circle(disp, self.tail, self.rail_line.segments[-1].sg.e, 3)

			disp.blit(self.start_text, self.rail_line.segments[0].sg.s)
			disp.blit(self.end_text, self.rail_line.segments[-1].sg.e.sub(Vector(0,10)))

			if self.rail_line.coal_start_pt != None:
				disp.blit(self.coal_start_text, self.rail_line.coal_start_pt)
				pygame.draw.circle(disp, coal_color, self.rail_line.coal_start_pt, 3)
				
			if self.rail_line.coal_end_pt != None:
				disp.blit(self.coal_end_text, self.rail_line.coal_end_pt.sub(Vector(0,10)))
				pygame.draw.circle(disp, coal_color, self.rail_line.coal_end_pt, 3)

			if self.rail_line.coal_stack != None:
				pygame.draw.circle(disp, coal_color, self.rail_line.coal_stack, 6)

class HelperSwitch:
	def __init__(self, switches, color = (73, 204, 84)):
		self.switches = switches
		self.color = color

	def draw_dir_line(self, disp, rail_line, from_start = True):
		if from_start:
			s = rail_line.segments[0].sg
			start_pt = s.s
			#end_pt = s.s.add(s.dir.scale(30))
			end_pt = s.e
		else:
			s = rail_line.segments[-1].sg
			start_pt = s.e
			#end_pt = s.e.add(s.rdir.scale(30))
			end_pt = s.s

		pygame.draw.line(disp, (0, 255, 0), start_pt, end_pt)
		pygame.draw.circle(disp, (2, 255, 0), end_pt, 2)

	def draw(self, disp):
		for sw in self.switches:
			rail_line = sw.lines[0][0]
			if sw.lines[0][1] == Switch.LINE_START:
				mid_pt = rail_line.segments[0].sg.s
			else:
				mid_pt = rail_line.segments[-1].sg.e
				
			pygame.draw.rect(disp, self.color, pygame.Rect(mid_pt.x - 3, mid_pt.y - 3, 6, 6), 1)

			for st in sw.states:
				if st[0].start_switch is sw:
					self.draw_dir_line(disp, st[0])
				else:
					self.draw_dir_line(disp, st[0], False)
				
				if st[1].start_switch is sw:
					self.draw_dir_line(disp, st[1])
				else:
					self.draw_dir_line(disp, st[1], False)
			
class HelperCoalStack:
	def __init__(self, pos, color = (200, 200, 200)):
		self.pos = pos
		self.color = color

	def draw(self, disp):
		pygame.draw.circle(disp, self.color, self.pos, 5)

class HelperActSwitch:
	def __init__(self, pos = None, color = (241, 238, 8)):
		self.pos = pos
		self.color = color
		self.visible = False

	def draw(self, disp):
		if self.visible:
			pygame.draw.circle(disp, self.color, self.pos, 5)

class Display:
	def __init__(self, display_surface, rail_lines):
		self.disp = display_surface
		self.helpers = []
		self.rail_lines = rail_lines
		
	def draw(self):
		self.disp.fill( (0,0,0) )

		pygame.draw.line(self.disp, (255, 255, 255), (0, scr_height*7/8), (scr_width, scr_height*7/8))

		for act_line in self.rail_lines:
			for act_segment in act_line.segments:
				if act_segment.arc == None:
					pygame.draw.line(self.disp, (255, 255, 255), act_segment.sg.s, act_segment.sg.e)
				else:
					act_segment.arc.draw(self.disp, (0, 0, 255))

		for hl in self.helpers:
			hl.draw(self.disp)

		pygame.display.flip()

def join_line_to_start_switch(act_line, switch_line, rail_lines, loop_lines, join_start_end = False):
	if switch_line.segments[0].arc == None:
		print ("Join to line")
		line2 = switch_line.segments[0].sg
	else:
		print ("Join to arc")
		line2 = Segment(switch_line.segments[0].sg.s, switch_line.segments[0].sg.s.add(switch_line.segments[0].arc.dir))

	if join_start_end:
		line1 = act_line.segments[0].sg
	else:
		line1 = act_line.segments[-1].sg
			
	mid_pt = line1.l.intersect(line2.l)

	if join_start_end:
		sg1 = Segment(line1.s, mid_pt)
	else:
		sg1 = Segment(line1.e, mid_pt)
	sg2 = Segment(mid_pt, switch_line.segments[0].sg.s)
	arc = sg1.calc_arc(sg2, sg2.length, False)

	if join_start_end:
		end_adjust = line1.rdir.scale(sg1.length - sg2.length)
		line1.s = line1.s.add(end_adjust)
	else:
		end_adjust = line1.dir.scale(sg1.length - sg2.length)
		line1.e = line1.e.add(end_adjust)
	line1.recalc()

	if join_start_end:
		arc.reverse()
		act_line.insert( RailSegment(Segment(line2.s, line1.s), arc) )
		switch_line.start_switch.add_line(act_line, Switch.LINE_START)
		#act_line.start_switch = switch_line.start_switch

		#act_line.start_switch.add_line(act_line, Switch.LINE_START)
	else:
		act_line.append( RailSegment(Segment(line1.e, line2.s), arc) )
		switch_line.start_switch.add_line(act_line, Switch.LINE_END)
		#act_line.end_switch = switch_line.start_switch

		#act_line.end_switch.add_line(act_line, Switch.LINE_END)

def join_line_to_end_switch(act_line, switch_line, rail_lines, loop_lines, join_start_end = False):
	if switch_line.segments[-1].arc == None:
		line2 = switch_line.segments[-1].sg
		print ("Join to line")
	else:
		print ("Join to arc")
		line2 = Segment(switch_line.segments[-1].sg.e.sub(switch_line.segments[-1].arc.rdir), switch_line.segments[-1].sg.e)
		#line2 = Segment(switch_line.segments[-1].sg.e, switch_line.segments[-1].sg.e.add(switch_line.segments[-1].arc.rdir))

	if join_start_end:
		#line1 = act_line.segments[0].sg
		line1 = Segment(act_line.segments[0].sg.e, act_line.segments[0].sg.s)
	else:
		line1 = act_line.segments[-1].sg
			
	mid_pt = line1.l.intersect(line2.l)
	sg1 = Segment(line1.e, mid_pt)
	#sg2 = Segment(mid_pt, switch_line.segments[-1].sg.s)
	sg2 = Segment(mid_pt, switch_line.segments[-1].sg.e)
	arc = sg1.calc_arc(sg2, sg2.length, False)

	end_adjust = line1.dir.scale(sg1.length - sg2.length)

	if join_start_end:
		act_line.segments[0].sg.s = act_line.segments[0].sg.s.add(end_adjust)
		act_line.segments[0].sg.recalc()
	else:
		line1.e = line1.e.add(end_adjust)
		line1.recalc()

	if join_start_end:
		arc.reverse()
		act_line.insert( RailSegment(Segment(line2.e, line1.e), arc) )
		#act_line.end_switch = switch_line.end_switch

		switch_line.end_switch.add_line(act_line, Switch.LINE_START)
	else:
		act_line.append( RailSegment(Segment(line1.e, line2.e), arc) )
		#act_line.end_switch = switch_line.end_switch

		switch_line.end_switch.add_line(act_line, Switch.LINE_END)

def join_lines(act_line, snap, rail_lines, loop_lines, switches, join_start_end = False):
	if snap.type == SnapInfo.SEGMENT_START and snap.line.segments.index(snap.segment) == 0 and snap.line.start_switch != None:
		print ("Join to start switch")
		join_line_to_start_switch(act_line, snap.line, rail_lines, loop_lines, join_start_end)
	
	elif snap.type == SnapInfo.SEGMENT_END and snap.line.segments.index(snap.segment) == len(snap.line)-1 and snap.line.end_switch != None:
		print ("Join to end switch")
		join_line_to_end_switch(act_line, snap.line, rail_lines, loop_lines, join_start_end)
	
	elif snap.type == SnapInfo.SEGMENT_START:
		print ("Join to segment start")
		# break the line into two pieces first
		break_pos = snap.line.segments.index(snap.segment)
		if snap.line in loop_lines:
			# if the destinationline is a loop then we don't create a new line just simply rearrange the loop
			snap.line.segments = snap.line.segments[break_pos:] + snap.line.segments[:break_pos]
			new_line = snap.line
			loop_lines.remove(snap.line)
		else:
			new_line = RailLine()
			new_line.segments = snap.line.segments[break_pos:]

			#new_line.end_switch = snap.line.end_switch
			if snap.line.end_switch != None:
				snap.line.end_switch.add_line(new_line, Switch.LINE_END)
				snap.line.end_switch.remove_line(snap.line, Switch.LINE_END)
				
			del snap.line.segments[break_pos:]
			rail_lines.append(new_line)
		
		sw = Switch(snap.line, Switch.LINE_END, new_line, Switch.LINE_START)
		switches.append(sw)
		#snap.line.end_swith = sw
		#new_line.start_switch = sw

		# ok, now join act_line into the new switch
		join_line_to_start_switch(act_line, new_line, rail_lines, loop_lines, join_start_end)

	elif snap.type == SnapInfo.LINE_MID:
		print ("Join to line mid")
		# break the line into two pieces first
		break_pos = snap.line.segments.index(snap.segment)
		if snap.line in loop_lines:
			# if the destinationline is a loop then we don't create a new line just simply rearrange the loop
			snap.line.segments = snap.line.segments[break_pos:] + snap.line.segments[:break_pos]
			new_line = snap.line
			loop_lines.remove(snap.line)
		else:
			new_line = RailLine()
			new_line.segments = snap.line.segments[break_pos:]
			
			#new_line.end_switch = snap.line.end_switch
			if snap.line.end_switch != None:
				snap.line.end_switch.add_line(new_line, Switch.LINE_END)
				snap.line.end_switch.remove_line(snap.line, Switch.LINE_END)

			del snap.line.segments[break_pos:]
			rail_lines.append(new_line)

		snap.line.segments.append( RailSegment( Segment(new_line.segments[0].sg.s, snap.pt)))
		
		if len(new_line.segments) > 0:
			new_line.segments[0].sg.s = snap.pt
			new_line.segments[0].sg.recalc()
		
		sw = Switch(snap.line, Switch.LINE_END, new_line, Switch.LINE_START)
		switches.append(sw)
		#snap.line.end_swith = sw
		#new_line.start_switch = sw
		
		# ok, now join act_line into the new switch
		join_line_to_start_switch(act_line, new_line, rail_lines, loop_lines, join_start_end)

def get_next_arc_segment_pair(arc_segments):
	if len(arc_segments) == 0:
		return None
		
	arc_seg = arc_segments.pop()
	while len(arc_segments) > 0 and (arc_seg[0].sg.length == 0 or arc_seg[1].sg.length == 0):
		arc_seg = arc_segments.pop()
	
	if arc_seg[0].sg.length != 0 and arc_seg[1].sg.length != 0:
		return arc_seg

def save_file(name, rail_lines, switches):
	xs = dest_x_res / scr_width
	ys = dest_y_res / scr_height
	
	empty_lines = []
	for act_line in rail_lines:
		if len(act_line) == 0:
			empty_lines.append(act_line)

	for act_line in empty_lines:
		rail_lines.remove(act_line)

	spawn_lines = []
	coal_positions = []
	
	f = open(name, "w")
	f.write('%d\n' % len(rail_lines))
	for act_line in rail_lines:
		f.write('%d' % len(act_line.segments))
		cpos = act_line.get_coal_positions()
		if cpos != None:
			#f.write(' %f %f' % cpos)
			f.write(' %f %f' % (cpos[0]*xs, cpos[1]*xs))
		f.write('\n')

		for act_seg in act_line.segments:
			if act_seg.arc != None:
				a = act_seg.arc
				#f.write('a %f %f %f %f %f\n' % (a.cp.x, a.cp.y, a.r, a.sang, a.eang))
				f.write('a %f %f %f %f %f\n' % (a.cp.x*xs, (scr_height - a.cp.y)*ys, a.r*xs, a.sang, a.eang))
			else:
				s = act_seg.sg
				#f.write('s %f %f %f %f\n' % (s.s.x, s.s.y, s.e.x, s.e.y))
				f.write('s %f %f %f %f\n' % (s.s.x*xs, (scr_height - s.s.y)*ys, s.e.x*xs, (scr_height - s.e.y)*ys))
				
		if act_line.spawn_line:
			spawn_lines.append(rail_lines.index(act_line))

		if act_line.coal_start_segment != None:
			#coal_positions.append((act_line.coal_stack[0], act_line.coal_stack[1], rail_lines.index(act_line)))
			coal_positions.append((act_line.coal_stack[0]*xs, (scr_height - act_line.coal_stack[1])*ys, rail_lines.index(act_line)))

	f.write('%d\n' % len(switches))
	for sw in switches:
		f.write('%d\n' % len(sw.states))
		for st in sw.states:
			line1_index = rail_lines.index(st[0])
			line2_index = rail_lines.index(st[1])
			
			if st[0].start_switch is sw:
				line1_type = 1
			else:
				line1_type = -1

			if st[1].start_switch is sw:
				line2_type = 1
			else:
				line2_type = -1

			f.write('%d %d %d %d\n' % (line1_index, line1_type, line2_index, line2_type))

	#f.write('%d\n' % len(spawn_lines))
	for sp in spawn_lines:
		f.write('%d ' % sp)

	f.write('\n%d\n' % len(coal_positions))
	for cp in coal_positions:
		f.write('%f %f %d\n' % cp) 

	f.close()

def load_file(name, rail_lines, switches):
	
	xs = scr_width / dest_x_res
	ys = scr_height / dest_y_res
	
	f = open(name, "r")

	lines = int( f.next().split()[0] )
	for i in range(lines):
		act_line = RailLine()
		rail_lines.append(act_line)
		
		l = f.next().split()
		segments = int(l[0])
		if len(l) > 1:
			# has coal positions
			pass

		for j in range(segments):
			l = f.next().split()
			if l[0] == 'a':
				a = Arc(Vector(float(l[1])*xs, scr_height - float(l[2])*ys), float(l[3])*xs, float(l[4]), float(l[5]))
				a.recalc()
				act_line.segments.append(RailSegment( Segment(a.s, a.e), a))
			else:
				start = Vector(float(l[1])*xs, scr_height - float(l[2])*ys)
				end = Vector(float(l[3])*xs, scr_height - float(l[4])*ys)
				act_line.segments.append(RailSegment( Segment(start, end) ))

	l = f.next().split()
	sws = int(l[0])
	for i in range(sws):
		sw = Switch()
		switches.append(sw)
		
		l = f.next().split()
		states = int(l[0])
		for j in range(states):
			l = f.next().split()
			
			line1 = rail_lines[int(l[0])]
			if int(l[1]) == 1:
				end = Switch.LINE_START
			else:
				end = Switch.LINE_END
			
			sw.add_line(line1, end)
			
			line2 = rail_lines[int(l[2])]
			if int(l[3]) == 1:
				end = Switch.LINE_START
			else:
				end = Switch.LINE_END
			
			sw.add_line(line2, end)
			sw.add_state(line1, line2)

	l = f.next().split()
	for i in l:
		rail_lines[int(i)].spawn_line = True
		
	l = f.next().split()
	coals = int(l[0])
	for i in range(coals):
		l = f.next().split()
		line = rail_lines[int(l[2])]
		line.coal_stack = Vector(float(l[0])*xs, scr_height - float(l[1])*ys)
		
	f.close()

State_new_line = 0
State_add_line_segment = 1
State_create_arc_segment = 2
State_merge_segments = 3
State_place_coal = 4
State_place_coal_end = 5
State_place_coal_stack = 6
State_switch_select = 7
State_switch_line1_select = 8
State_switch_line2_select = 9
State_spawn_line_select = 10

Add_mode_append = 0
Add_mode_insert = 1

# Snap distances are squared values
Snap_dist_pt = 150
Snap_dist_line = 100
Snap_dist_arc = 100

psyco.profile()

def main():
	pygame.init()
	display_surface = init_display()
	clock = pygame.time.Clock()

	act_line = RailLine()
	rail_lines = [act_line]
	loop_lines = []

	switches = []

	disp = Display(display_surface, rail_lines)

	new_line_helper = HelperLine()
	disp.helpers.append(new_line_helper)
	
	#snap_point_helper = HelperPoint()
	snap_point_helper = HelperSnap()
	disp.helpers.append(snap_point_helper)

	rail_line_helper = HelperRailLine()
	disp.helpers.append(rail_line_helper)

	switch_helper = HelperSwitch(switches)
	disp.helpers.append(switch_helper)

	state = State_new_line
	segment_add_mode = Add_mode_append

	while True:
		clock.tick(20)
		
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				sys.exit()

			elif event.type == pygame.KEYDOWN and event.key == K_ESCAPE:
				sys.exit()

			elif event.type == pygame.KEYDOWN and event.key == K_c:
				# place coal
				if state == State_new_line:
					state = State_place_coal

			elif event.type == pygame.KEYDOWN and event.key == K_w:
				if state == State_new_line:
					act_switch_helper = HelperActSwitch()
					disp.helpers.append(act_switch_helper)
					state = State_switch_select

			elif event.type == pygame.KEYDOWN and event.key == K_s:
				if state == State_new_line:
					state = State_spawn_line_select

			elif event.type == pygame.KEYDOWN and event.key == K_F2:
				save_file("mittomen.txt", rail_lines, switches)
			
			elif event.type == pygame.KEYDOWN and event.key == K_F5:
				load_file("mittomen.txt", rail_lines, switches)

			elif event.type == pygame.MOUSEBUTTONDOWN:
				buttons = pygame.mouse.get_pressed()
				if buttons[0]: # left button pressed
					if state == State_new_line:
						prev_pt = pygame.mouse.get_pos()
						if snap != None:
							if snap.type == SnapInfo.SEGMENT_END and not snap.line in loop_lines and snap.line.segments.index(snap.segment) == len(snap.line)-1:
								# we are continuing a non closed line
								#rail_lines.remove(act_line)
								act_line = snap.line
								prev_pt = snap.pt
								segment_add_mode = Add_mode_append
							elif snap.type == SnapInfo.SEGMENT_START and not snap.line in loop_lines and snap.line.segments.index(snap.segment) == 0:
								#we are inserting new segments into the start of an existing line
								#rail_lines.remove(act_line)
								act_line = snap.line
								prev_pt = snap.pt
								segment_add_mode = Add_mode_insert
							else:
								act_line = RailLine()
								rail_lines.append(act_line)
								
								segment_add_mode = Add_mode_append
						else:
							act_line = RailLine()
							rail_lines.append(act_line)
							
							segment_add_mode = Add_mode_append
							
						new_line_helper.show(prev_pt)
						state = State_add_line_segment

					elif state == State_add_line_segment:
						mpos = t2vec(pygame.mouse.get_pos())			
						arc_segments = []
						segments_merged = False
						segments_joined = False
						if segment_add_mode == Add_mode_append:
							if snap != None:
								if snap.type == SnapInfo.SEGMENT_START and snap.line.segments.index(snap.segment) == 0 and snap.line.start_switch == None:
									# it;s a start of a line with no switch
									act_line.append(RailSegment( Segment(t2vec(prev_pt), snap.pt)))
									if snap.line is act_line:
										# it will be a loop
										loop_lines.append(act_line)
										arc_segments.append([ act_line.segments[-1], act_line.segments[0] ])
										if len(act_line) > 1:
											arc_segments.append([ act_line.segments[-2], act_line.segments[-1] ])
										new_state = State_new_line # begin a new line after finishing the loop
									else:
										#it's a start of an another line so merge them
										arc_segments.append([ act_line.segments[-1], snap.line.segments[0] ])
										if len(act_line) > 1:
											arc_segments.append([ act_line.segments[-2], act_line.segments[-1] ])
										snap.line.segments[:0] = act_line.segments
										
										if act_line.start_switch != None:
											print("Actual line has a start switch so replacing the references to it")
											act_line.start_switch.replace_references(act_line, snap.line, Switch.LINE_START)

										rail_lines.remove(act_line)
										act_line = snap.line
										new_state = State_new_line
										segments_merged = True
										
								elif snap.type == SnapInfo.SEGMENT_END and snap.line.end_switch == None:
									# it's the end of the segment so we merge them but the actual line must be reversed
									act_line.append(RailSegment( Segment(t2vec(prev_pt), snap.pt)))
									act_line.reverse()
									if len(act_line) > 1:
										arc_segments.append([ act_line.segments[0], act_line.segments[1] ])
									arc_segments.append([ snap.line.segments[-1], act_line.segments[0] ])
									snap.line.segments.extend(act_line.segments)
									rail_lines.remove(act_line)
									act_line = snap.line
									new_state = State_new_line
									segments_merged = True
								else:
									# anything else is a line join
									if len(act_line) > 0:
										print("Append mode join")
										join_lines(act_line, snap, rail_lines, loop_lines, switches)
										segments_joined = True
									new_state = State_new_line
							else:
								act_line.append(RailSegment( Segment(t2vec(prev_pt), mpos)))
								if len(act_line) > 1:
									arc_segments.append([ act_line.segments[-2], act_line.segments[-1] ])
								new_state = State_add_line_segment

							arc_seg = get_next_arc_segment_pair(arc_segments)

							if arc_seg != None:
								new_line_helper.hide()
								act_arc = Arc(t2vec(prev_pt), 1, 0, 0)
								arc_segment_helper = HelperArc( act_arc )
								disp.helpers.append(arc_segment_helper)
								state = State_create_arc_segment
							else:
								if new_state == State_new_line:
									act_line = None
									new_line_helper.hide()
									#disp.helpers.remove(new_line_helper)
									#act_line = RailLine()
									#rail_lines.append(act_line)
									state = new_state
								else:
									prev_pt = mpos
									new_line_helper.s = prev_pt
								
						else: # insert mode
							if snap != None:
								if snap.type == SnapInfo.SEGMENT_END and snap.line.end_switch == None:
									# It's the end of a line with no switch
									act_line.insert(RailSegment(Segment(snap.pt, t2vec(prev_pt))))
									if snap.line is act_line:
										# it will be a loop
										loop_lines.append(act_line)
										arc_segments.append([ act_line.segments[-1], act_line.segments[0] ])
										if len(act_line) > 1:
											arc_segments.append([ act_line.segments[0], act_line.segments[1] ])
										new_state = State_new_line # begin a new line after finishing the loop
									else:
										#it's the end of an another segment so merge them
										arc_segments.append([ snap.line.segments[-1], act_line.segments[0] ])
										if len(act_line) > 1:
											arc_segments.append([ act_line.segments[0], act_line.segments[1] ])
										snap.line.segments.extend(act_line.segments)
										
										if act_line.end_switch != None:
											act_line.end_switch.replace_references(act_line, snap.line, Switch.LINE_END)
											
										rail_lines.remove(act_line)
										act_line = snap.line
										new_state = State_new_line
										segments_merged = True
										
								elif snap.type == SnapInfo.SEGMENT_START and snap.line.segments.index(snap.segment) == 0 and snap.line.start_switch == None:
									# it's the start of the a line so we merge them but the actual line must be reversed
									act_line.insert(RailSegment( Segment(snap.pt, t2vec(prev_pt))))
									act_line.reverse()
									arc_segments.append([ act_line.segments[-1], snap.line.segments[0] ])
									if len(act_line) > 1:
										arc_segments.append([ act_line.segments[-2], act_line.segments[-1] ])
									act_line.segments.extend(snap.line.segments)
									rail_lines.remove(snap.line)
									new_state = State_new_line
									segments_merged = True
								else:
									# anything else is a line join
									if len(act_line) > 0:
										print("Insert mode join")
										join_lines(act_line, snap, rail_lines, loop_lines, switches, True)
										segments_joined = True
									new_state = State_new_line
							else:
								act_line.insert(RailSegment(Segment(mpos, t2vec(prev_pt))))
								if len(act_line) > 1:
									arc_segments.append([ act_line.segments[0], act_line.segments[1] ])
								new_state = State_add_line_segment
							
							arc_seg = get_next_arc_segment_pair(arc_segments)

							if arc_seg != None:
								new_line_helper.hide()
								act_arc = Arc(t2vec(prev_pt), 1, 0, 0)
								arc_segment_helper = HelperArc( act_arc )
								disp.helpers.append(arc_segment_helper)
								state = State_create_arc_segment
							else:
								if new_state == State_new_line:
									act_line = None
									new_line_helper.hide()
									#disp.helpers.remove(new_line_helper)
									#act_line = RailLine()
									#rail_lines.append(act_line)
									state = new_state
								else:
									prev_pt = mpos
									new_line_helper.s = prev_pt

					elif state == State_create_arc_segment:
						mpos = pygame.mouse.get_pos()
						insert_pos = act_line.segments.index(arc_seg[1])
						act_line.segments.insert(insert_pos, RailSegment( Segment(act_arc.s, act_arc.e), act_arc) )
						arc_seg[0].sg.e = act_arc.s
						arc_seg[0].sg.recalc()

						if arc_seg[0].sg.length == 0:
							act_line.segments.remove(arc_seg[0])

						arc_seg[1].sg.s = act_arc.e
						arc_seg[1].sg.recalc()

						if arc_seg[1].sg.length == 0:
							act_line.segments.remove(arc_seg[1])

						new_arc_seg = get_next_arc_segment_pair(arc_segments)
						
						if new_arc_seg == None:
							disp.helpers.remove(arc_segment_helper)
							if segment_add_mode == Add_mode_append:
								prev_pt = arc_seg[1].sg.e
							else:
								prev_pt = arc_seg[0].sg.s

							state = new_state
							#if state == State_new_line:
								#act_line = RailLine()
								#rail_lines.append(act_line)
							if state == State_add_line_segment:
								new_line_helper.show(prev_pt, mpos)
						else:
							arc_seg = new_arc_seg
							
					elif state == State_place_coal:
						if snap != None:
							act_line = snap.line
							
							act_line.coal_start_segment = snap.segment
							act_line.coal_start_pt = snap.pt
							if snap.segment.arc != None:
								act_line.coal_start_ang = snap.angle

							state = State_place_coal_end

					elif state == State_place_coal_end:
						if snap != None and snap.line is act_line:
							act_line.coal_end_segment = snap.segment
							act_line.coal_end_pt = snap.pt
							if snap.segment.arc != None:
								act_line.coal_end_ang = snap.angle
							
							start_index = act_line.segments.index(act_line.coal_start_segment)
							end_index = act_line.segments.index(snap.segment)
							d1 = act_line.coal_start_pt.dist2(act_line.coal_start_segment.sg.s)
							d2 = act_line.coal_end_pt.dist2(act_line.coal_start_segment.sg.s)
							if start_index > end_index or (start_index == end_index and d1 > d2):
								act_line.coal_start_segment, act_line.coal_end_segment = act_line.coal_end_segment, act_line.coal_start_segment
								act_line.coal_start_pt, act_line.coal_end_pt = act_line.coal_end_pt, act_line.coal_start_pt
								act_line.coal_start_ang, act_line.coal_end_ang = act_line.coal_end_ang, act_line.coal_start_ang
							#act_line = None
							coal_stack_helper = HelperCoalStack(pygame.mouse.get_pos())
							disp.helpers.append(coal_stack_helper)
							state = State_place_coal_stack
							#state = State_new_line
							
					elif state == State_place_coal_stack:
						act_line.coal_stack = pygame.mouse.get_pos()
						disp.helpers.remove(coal_stack_helper)
						act_line = None
						state = State_new_line

					elif state == State_switch_select:
						if snap != None and snap.type in [SnapInfo.SEGMENT_START, SnapInfo.SEGMENT_END]:
							if snap.type == SnapInfo.SEGMENT_START and snap.line.start_switch != None and snap.line.segments[0] is snap.segment:
								act_switch = snap.line.start_switch

								#act_switch_helper = HelperActSwitch(snap.pt)
								act_switch_helper.pos = snap.pt
								act_switch_helper.visible = True
								
								snap_point_helper.visible = False
								state = State_switch_line1_select
							elif snap.type == SnapInfo.SEGMENT_END and snap.line.end_switch != None:
								act_switch = snap.line.end_switch
								
								act_switch_helper.pos = snap.pt
								act_switch_helper.visible = True
								
								snap_point_helper.visible = False
								state = State_switch_line1_select

					elif state == State_switch_line1_select:
						if snap != None:
							if snap.line.start_switch is act_switch or snap.line.end_switch is act_switch:
								print ("First line selection OK")
								switch_line1 = snap.line
								state = State_switch_line2_select
							else:
								print ("The selected line doesn't belong to the switch")
					
					elif state == State_switch_line2_select:
						if snap != None:
							if (snap.line.start_switch is act_switch or snap.line.end_switch is act_switch) and not snap.line is switch_line1:
								if not [switch_line1, snap.line] in act_switch.states and not [snap.line, switch_line1] in act_switch.states:
									print ("Second line selection OK, switch positon added")
									act_switch.add_state(switch_line1, snap.line)
									state = State_switch_line1_select
								else:
									print ("The given state is already in the list")
							else:
								print("The selected line doesn't belong to the switch or you have selected the line twice")
									
					elif state == State_spawn_line_select:
						if snap != None:
							print("Spawn line selected")
							snap.line.spawn_line = True
							state = State_new_line

				if buttons[2]: # right click
					if state == State_add_line_segment:
						new_line_helper.hide()

						#act_line = RailLine()
						#rail_lines.append(act_line)
						act_line = None
						state = State_new_line

					elif state in [State_switch_select, State_switch_line1_select, State_switch_line2_select]:
						disp.helpers.remove(act_switch_helper)
						new_line_helper.hide()
						act_line = None
						state = State_new_line
	
			elif event.type == pygame.MOUSEMOTION:
				if state == State_new_line:
					mpos = pygame.mouse.get_pos()
					snap = find_snap_point(rail_lines, t2vec(mpos), Snap_dist_pt, Snap_dist_line, Snap_dist_arc)
					if snap != None:
						#snap_point_helper.pos = snap.pt
						snap_point_helper.snap = snap
						snap_point_helper.visible = True

						rail_line_helper.rail_line = snap.line
						rail_line_helper.visible = True
					else:
						snap_point_helper.visible = False
						rail_line_helper.visible = False
					
				elif state == State_add_line_segment:
					mpos = pygame.mouse.get_pos()
					new_line_helper.e = pygame.mouse.get_pos()

					snap = find_snap_point(rail_lines, t2vec(mpos), Snap_dist_pt, Snap_dist_line, Snap_dist_arc)
					if snap != None:
						#snap_point_helper.pos = snap.pt
						snap_point_helper.snap = snap
						snap_point_helper.visible = True
						
						rail_line_helper.rail_line = snap.line
						rail_line_helper.visible = True
						
					else:
						snap_point_helper.visible = False
						rail_line_helper.visible = False
					
				elif state == State_create_arc_segment:
					mpos = t2vec(pygame.mouse.get_pos())
					dist = mpos.dist(arc_seg[0].sg.e)
					act_arc = arc_seg[0].sg.calc_arc(arc_seg[1].sg,dist)
					arc_segment_helper.a = act_arc
				
				elif state in [State_place_coal, State_place_coal_end, State_switch_select]:
					mpos = pygame.mouse.get_pos()
					snap = find_snap_point(rail_lines, t2vec(mpos), Snap_dist_pt, Snap_dist_line, Snap_dist_arc)
					if snap != None:
						#snap_point_helper.pos = snap.pt
						snap_point_helper.snap = snap
						snap_point_helper.visible = True

						rail_line_helper.rail_line = snap.line
						rail_line_helper.visible = True
					else:
						snap_point_helper.visible = False
						rail_line_helper.visible = False
						
				elif state in [State_switch_line1_select, State_switch_line2_select, State_spawn_line_select]:
					mpos = pygame.mouse.get_pos()
					snap = find_snap_point(rail_lines, t2vec(mpos), Snap_dist_pt, Snap_dist_line, Snap_dist_arc)
					if snap != None:
						#snap_point_helper.pos = snap.pt
						#snap_point_helper.visible = True

						rail_line_helper.rail_line = snap.line
						rail_line_helper.visible = True
					else:
						#snap_point_helper.visible = False
						rail_line_helper.visible = False


				elif state == State_place_coal_stack:
					coal_stack_helper.pos = pygame.mouse.get_pos()
	
		disp.draw()

main()

