import sys, math, pygame
from pygame.locals import *

scr_width = 320
scr_height = 240

def init_display():
	return pygame.display.set_mode((scr_width, scr_height))

class Vector:
	def __init__(self, x, y):
		self.x = x
		self.y = y

	def __len__(self):
		return 2

	def __getitem__(self, n):
		if n == 0:
			return self.x
		else:
			return self.y

	def len2(self):
		return self.x * self.x + self.y*self.y

	def len(self):
		return math.sqrt(self.len2())

	def dist2(self, v):
		dx, dy = v.x - self.x, v.y - self.y
		return dx*dx + dy*dy

	def dist(self, v):
		return math.sqrt(self.dist2(v))

	def add(self, v):
		return Vector(self.x + v.x, self.y + v.y)
	
	def sub(self, v):
		return Vector(self.x - v.x, self.y - v.y)

	def reverse(self):
		return Vector(-self.x, -self.y)

	def dot_product(self, v):
		return self.x * v.x + self.y * v.y

	def scale(self, factor):
		return Vector(self.x * factor, self.y * factor)

#	def v2tup(self):
#		return (self.x, self.y)

def t2vec(tuple):
	return Vector(tuple[0], tuple[1])

class Line:
	def __init__(self, normal, pt):
		self.a = normal.x
		self.b = normal.y
		self.c = -self.a*pt.x - self.b*pt.y
		
	def intersect(self, l):
		a, b, c = self.a, self.b, self.c
		d, e, f = l.a, l.b, l.c

		return Vector((e*c - b*f) / (b*d - e*a), (d*c - a*f) / (e*a - b*d))

class Arc:
	def __init__(self, center, radius, start_ang, end_ang, start = Vector(0,0), end = Vector(0,0)):
		self.cp = center
		self.r = radius
		self.sang = start_ang
		self.eang = end_ang
		self.s = start
		self.e = end

class Segment:
	def __init__(self, start, end):
		self.s = start
		self.e = end
		self.se = end.sub(start)
		self.es = start.sub(end)
		self.l = Line(Vector(-self.se.y, self.se.x), self.s)
		self.length = self.se.len()
		self.dir =  self.se.scale(1/self.length)

	def sees(self, v):
		sv, ev = v.sub(self.s), v.sub(self.e)
		return self.se.dot_product(sv) >= 0 and self.es.dot_product(ev) >= 0

	def project(self, v, crop_to_end = False):
		if self.sees(v):
			l = Line(self.dir, v)
			return self.l.intersect(l)
		elif crop_to_end:
			sv, ev = v.sub(self.s), v.sub(self.e)
			if self.se.dot_product(sv) < 0:
				return self.s
			else:
				return self.e

	def calc_arc(self, end_segment, dist):
		dist = min(dist, self.length, end_segment.length)
		
		p1 = self.dir.scale(-dist).add(self.e)
		p2 = end_segment.dir.scale(dist).add(end_segment.s)
		midpt = self.e
		
		l1 = Line(self.dir, p1)
		l2 = Line(end_segment.dir, p2)
		cp = l1.intersect(l2)
		r = cp.dist(p1)
		
		sa = abs(p2.y - cp.y) / r
		if sa > 1:
			sa = 1
	
		if p2.x >= cp.x and p2.y <= cp.y:
			alpha = math.asin(sa)
		elif p2.x <= cp.x and p2.y <= cp.y:
			alpha = math.pi - math.asin(sa)
		elif p2.x <= cp.x and p2.y >= cp.y: 
			alpha = math.pi + math.asin(sa)
		else:
			alpha = 2*math.pi - math.asin(sa)
			
		sb = abs(p1.y - cp.y) / r
		if sb > 1:
			sb = 1
			
		if p1.x >= cp.x and p1.y <= cp.y:
			beta = math.asin(sb)
		elif p1.x <= cp.x and p1.y <= cp.y:
			beta = math.pi - math.asin(sb)
		elif p1.x <= cp.x and p1.y >= cp.y: 
			beta = math.pi + math.asin(sb)
		else:
			beta = 2*math.pi - math.asin(sb)
	
		# Only swap them for drawing!!!
		if alpha > beta:
			alpha, beta = beta, alpha

		if beta - alpha > math.pi:
			alpha, beta = beta, alpha + 2*math.pi

		return Arc(cp, r, alpha, beta, p1, p2)

class RailSegment:
	def __init__(self, segment):
		self.sg = segment
		self.arc = None

class RailLine:
	def __init__(self):
		self.segments = []

	def len(self):
		return len(self.segments)

class Switch:
	LINE_START = 0
	LINE_END = 1
	def __init__(self, line1, end1, line2, end2):
		self.lines = [ [line1, end1], [line2, end2] ]

	def add_line(self, line, end):
		self.lines.append([line, end])

class SnapInfo:
	POINT = 0
	LINE = 1
	ARC = 2
	
	SEGMENT_START = 0
	SEGMENT_END = 1

	def __init__(self, line, segment, dist, type, pt):
		self.line = line
		self.segment = segment
		self.dist = dist
		self.type = type
		self.pt = pt
		
def find_snap_point(rail_lines, reference_pt, pt_snap_dist, line_snap_dist, arc_snap_dist, exclude_lines = []):
	min_dist = -1

	# search for points to snap to
	if pt_snap_dist != -1:
		for act_line in rail_lines:
			if act_line in exclude_lines:
				continue
			if len(act_line) > 0:
				for act_segment in act_line:
					act_dist = act_segment.sg.s.dist2(reference_pt)
					if act_dist < pt_snap_dist and (min_dist == -1 or min_dist > act_dist):
						min_dist = act_dist
						min_segment = act_segment
						min_line = act_line

				act_dist = act_line[-1].sg.e.dist2(reference_pt)
				if act_dist < pt_snap_dist and (min_dist == -1 or min_dist > act_dist):
					min_dist = act_dist
					min_segment = act_segment
					min_line = act_line
					alignment = SnapInfo.SEGMENT_END
				else:
					alignment = SnapInfo.SEGMENT_START
	
		if min_dist != -1:
			return SnapInfo(min_line, min_segment, min_dist, SnapInfo.POINT, alignment)
#			return (min_line, min_segment, min_dist)

	# search for lines to snap to
	if line_snap_dist != -1:
		for act_line in rail_lines:
			if act_line in exclude_lines:
				continue
			for act_segment in act_line:
				pr_pt = act_segment.sg.project(reference_pt)
				if pr_pt != None:
					act_dist = pr_pt.dist2(reference_pt)
					if act_dist < pt_snap_dist and (min_dist == -1 or min_dist > act_dist):
						min_dist = act_dist
						min_segment = act_segment
						min_line = act_line
						min_pt = pr_pt

		if min_dist != -1:
			return SnapInfo(min_line, min_segment, min_dist, SnapInfo.LINE, min_pt)
#			return (min_line, min_segment, min_dist, min_pt)

class HelperPoint:
	def __init__(self, pos = Vector(0,0), visible = False, color = (255, 0, 0)):
		self.pos = pos
		self.color = color
		self.visible = visible

	def draw(self, disp):
		if self.visible:
			pygame.draw.circle(disp, self.color, self.pos, 2)

class HelperLine:
	def __init__(self, start, end, color = (255, 0, 0)):
		self.s = start
		self.e = end
		self.c = color

	def draw(self, disp):
		pygame.draw.line(disp, self.c, self.s, self.e)

class HelperArc:
	def __init__(self, arc, color = (0, 255, 0)):
		self.a = arc
		self.c = color
		self.enabled = True

	def draw(self, disp):
		if self.enabled:
			r = self.a.r
			if r < 1:
				r = 1
			rect = pygame.Rect(self.a.cp.x - r, self.a.cp.y - r, r*2, r*2)
			pygame.draw.arc(disp, self.c, rect, self.a.sang, self.a.eang)

class Display:
	def __init__(self, display_surface, rail_lines):
		self.disp = display_surface
		self.helpers = []
		self.rail_lines = rail_lines
		
	def draw(self):
		self.disp.fill( (0,0,0) )

		for act_line in self.rail_lines:
			prev_segment = None
			for act_segment in act_line:
				if prev_segment != None and prev_segment.arc != None:
					#start = prev_segment.arc.e.v2tup()
					start = prev_segment.arc.e
					pygame.draw.line(self.disp, (0, 0, 255), act_segment.sg.s, start)
				else:
					start = act_segment.sg.s

				if act_segment.arc != None:
					a = act_segment.arc
					r = a.r
					if r < 1:
						r = 1
					rect = pygame.Rect(a.cp.x - r, a.cp.y - r, r*2, r*2)
					pygame.draw.arc(self.disp, (255, 255, 255), rect, a.sang, a.eang)
					end = a.s
					pygame.draw.line(self.disp, (0, 0, 255), end, act_segment.sg.e)
				else:
					end = act_segment.sg.e
					
				pygame.draw.line(self.disp, (255, 255, 255), start, end)
				prev_segment = act_segment

		for hl in self.helpers:
			hl.draw(self.disp)

		pygame.display.flip()

State_new_line = 0
State_add_line_segment = 1
State_create_arc_segment = 2
State_create_closing_arc_segment1 = 3
State_create_closing_arc_segment2 = 4

# Snap distances are squared values
Snap_dist_pt = 150
Snap_dist_line = 100
Snap_dist_arc = 100

def main():
	pygame.init()
	display_surface = init_display()
	clock = pygame.time.Clock()

	act_line = []
	rail_lines = [act_line]
	loop_lines = []

	disp = Display(display_surface, rail_lines)
	snap_point_helper = HelperPoint()
	disp.helpers.append(snap_point_helper)

	state = State_new_line

	while True:
		clock.tick(20)
		
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				sys.exit()

			elif event.type == pygame.KEYDOWN and event.key == K_ESCAPE:
				sys.exit()

			elif event.type == pygame.MOUSEBUTTONDOWN:
				buttons = pygame.mouse.get_pressed()
				if buttons[0]: # left button pressed
					if state == State_new_line:
						prev_pt = pygame.mouse.get_pos()
						new_line_helper = HelperLine(prev_pt, prev_pt)
						disp.helpers.append(new_line_helper)
						state = State_add_line_segment

					elif state == State_add_line_segment:
						mpos = t2vec(pygame.mouse.get_pos())
						if snap != None:
							if snap.line == act_line and snap.type == SnapInfo.POINT and act_line.index(snap.segment) == 0 and snap.pt == SnapInfo.SEGMENT_START:
								pos = snap.segment.sg.s
								new_state = State_create_closing_arc_segment1
								loop_lines.append(act_line)
							else:
								pos = mpos
								new_state = State_create_arc_segment
						else:
							pos = mpos
							new_state = State_create_arc_segment
							
						act_line.append(RailSegment( Segment(t2vec(prev_pt), pos)))
						
						if len(act_line) > 1:
							# there are at least 2 segments so create an arc between them
							disp.helpers.remove(new_line_helper)
							act_arc = Arc(t2vec(prev_pt), 1, 0, 0)
							arc_segment_helper = HelperArc( act_arc )
							disp.helpers.append(arc_segment_helper)
							arc_seg1 = act_line[-2]
							arc_seg2 = act_line[-1]
							state = new_state
						else:
							prev_pt = mpos
							new_line_helper.s = prev_pt

					elif state == State_create_arc_segment:
						mpos = pygame.mouse.get_pos()
						arc_seg1.arc = act_arc
						disp.helpers.remove(arc_segment_helper)
						prev_pt = arc_seg2.sg.e
						new_line_helper = HelperLine(prev_pt, mpos)
						disp.helpers.append(new_line_helper)
						state = State_add_line_segment
						
					elif state == State_create_closing_arc_segment1:
						mpos = pygame.mouse.get_pos()
						arc_seg1.arc = act_arc
						act_arc = Arc(t2vec(prev_pt), 1, 0, 0)
						arc_segment_helper.a = act_arc
						arc_seg1 = act_line[-1]
						arc_seg2 = act_line[0]
						prev_pt = arc_seg2.sg.e
						state = State_create_closing_arc_segment2

					elif state == State_create_closing_arc_segment2:
						mpos = pygame.mouse.get_pos()
						arc_seg1.arc = act_arc
						disp.helpers.remove(arc_segment_helper)
						prev_pt = arc_seg2.sg.e

						act_line = []
						rail_lines.append(act_line)
						state = State_new_line
	
			elif event.type == pygame.MOUSEMOTION:
				if state == State_add_line_segment:
					mpos = pygame.mouse.get_pos()
					new_line_helper.e = pygame.mouse.get_pos()

					snap = find_snap_point(rail_lines, t2vec(mpos), Snap_dist_pt, Snap_dist_line, Snap_dist_arc)
					if snap != None:
						if snap.type == SnapInfo.LINE:
							snap_point_helper.pos = snap.pt
							snap_point_helper.visible = True
						elif snap.type == SnapInfo.POINT:
							if snap.pt == SnapInfo.SEGMENT_START:
								snap_point_helper.pos = snap.segment.sg.s
							else:
								snap_point_helper.pos = snap.segment.sg.e
					else:
						snap_point_helper.visible = False
					
				elif state in [State_create_arc_segment, State_create_closing_arc_segment1, State_create_closing_arc_segment2]:
					mpos = t2vec(pygame.mouse.get_pos())
					dist = mpos.dist(arc_seg1.sg.e)
					act_arc = arc_seg1.sg.calc_arc(arc_seg2.sg,dist)
					arc_segment_helper.a = act_arc
						
		disp.draw()

main()

