import os
import pygame
import gamestate_fileload
import gamestate_syncpackage
import gamestate_collision
import gamestate_events
from OpenGL.GL import *
import render
import vector_op as vec
from math import sin


from constants import *

#import timing

class Gamestate:

	def __init__(self):
		# Game info
		self.maxpoints = 300
			# Points to race for
		self.mapname = ""
			# No map by default
		self.players = []
			# Reference to Player objects
		self.lines = []
			# Reference to Line objects
		self.switches = []
			# Reference to Switch objects
		self.coalbases = {}
			# Contains line_ref -> (coal_posx, coal_posy) pairs
		self.spawnlines = []
			# Contains the line_refs of the spawning lines 
		
		self.clock = pygame.time.Clock()
			# Timer
		self.time = 0
			# game time in msec
		self.game = False
			# game is running
			
		self.isServer = True
			# if the local host is the server
			# this is the case if there is no network, too
		self.nextUpdate = 0
			# When to send next sync boost
		self.nextFullUpdate = 0
			# When to send full sync
			
		# Player info
		self.own_id = None;
		self.ownplayer = None;
			# Own player id, ref	
		
		# Misc info
		self.colorIndex = 0
			# which colors are used
		self.speedCorrection = 1.0
			# network speed correction
		
		### Attached functions
		# loadMapFile()
		# render()

	def reset(self):
		pass
	
	# Sets own player
	def setOwnPlayer(self, player):
		self.ownplayer = player
		self.own_id = player.id
		
		
	# Returns elapsed time, and accounts general time
	def tick(self):
		delta_msec = self.clock.tick()
		self.time += delta_msec
		return delta_msec
		
	# server time iteration function, gives AP, score, etc..
	def serverTime(self, delta_msec):
		dt = delta_msec / 1000.0
		for pl in self.players:
			# Distribute Action Points
			pl.action_points = min(pl.action_points + dt * DELTA_AP, MAX_ACTION_POINTS)
			
			tr = pl.train
			# Give score, depending on the number of cars pulled
			pl.score += dt * tr.cars * DELTA_SCORE_PER_CAR
			
			# Give coal, if coal is running low
			if tr.maxforce < MAX_FORCE/5.0 and tr.coal == 0:
				tr.coal += 1
	
	
	#
	# To be called once immediately before main loop
	#	
	def startGame(self):
		self.tick()
		self.game = True
		
		if self.isServer:
			# Time network update
			self.nextFullUpdate = 0
		
			
	#
	# Server loop
	# net is False if no network is used
	# refernce to Net class otherwise
	#
	def doTime(self, net):
		delta = self.tick()
		dt = delta/1000.0
		
		#################
		## Local events
		#################
		self.game = self.handleEvents()
		
		###########################
		## Network phase one
		###########################
		if net:
			if self.isServer:
				# Receive events
				net.serverRead()
				##########################
				## Network update phase
				##########################
			
				if self.time >= self.nextFullUpdate:
					# Full sync is to be sent
					net.sendUpdate(True)
					# Schedule next updates
					self.nextUpdate = self.time + NET_UPDATE_IVAL
					self.nextFullUpdate = self.time + NET_FULL_UPDATE_IVAL
					
				elif self.time >= self.nextUpdate:
					# Update required
					net.sendUpdate()
					# Schedule next update
					self.nextUpdate = self.time + NET_UPDATE_IVAL
					
			else:
				# Receive updates
				net.clientRead()
		
		#####################
		### Model phase
		#####################
		if self.isServer:
			# Switch state handling is for the server, too
			for sw in self.switches:
				sw.beginRound()
				sw.refresh()
			
		# Clients also do approximation of movement
		for pl in self.players:
			pl.train.move(dt)
				
		if self.isServer:
			# Coillision and point distribution is made by server
			self.checkCollision()
			self.serverTime(delta)
		
			# Close switch round
			for sw in self.switches:
				sw.endRound()
		
		##################
		## Be nice
		##################
		pygame.time.wait(5)
		
		
		
	# Returns the next color to use
	def nextColor(self):
		self.colorIndex += 1
		return PLAYER_COLORS[self.colorIndex - 1]
		
		
	# Add a line to the railway
	def addLine(self, ln):
		self.lines.append(ln)

	# Adds a switch to the railway
	# the switch should be pre-inited
	# with the states registered in it
	def addSwitch(self, sw):
		self.switches.append(sw)

	# Adds a player to the game
	# the player should be pre-inited appropriately
	def addPlayer(self, player, local = False):
		self.players.append(player)
		if local:
			self.setOwnPlayer(player)
	
	# Put the player in the appropriate position of the list
	def addPlayerAt(self, pos, player, local = False):
		# Extend size
		while len(self.players) <= player.id:
			self.players.append(None)
		
		# Put to position
		self.players[player.id] = player
		
		# Register local
		if local:
			# However, this interface is never used for local...
			self.setOwnPlayer(player)
			
	
	# Puts a train to an empty line
	# Preferrably a special line
	def placeTrain(self, train):
		train.defaultPos()
		for tries in (self.spawnlines, self.lines):
			for line in tries:
				# Check if line is empty
				empty = True
				for pl in self.players:
					tr = pl.train
					if (tr.head[LINE_REF_IDX] == line) or (tr.tail[LINE_REF_IDX] == line):
						# Not empty
						empty = False
						break
				
				if empty:
					# Empty, place there, if applicable
					tr_len = train.getLength()
					if  tr_len + 2*SWITCH_RANGE < line.length:
						#print "Placing on", self.lines.index(line), "tr_len", tr_len, "line_len", line.length, 'line=', line
						# Long enough
						train.head = (line, (line.length - tr_len)/2 + tr_len, 1)
						train.alignToHead()
						return
		# O ooh
		raise Exception("Could not place train!!!!")
		
		
	# Puts all trains to an empty line 
	# preferrably the special lines
	def placeTrains(self):
		for pl in self.players:
			self.placeTrain(pl.train)

#
# Graphical gamestate class
#
class GGamestate(Gamestate):
	
	def __init__(self, resources):
		Gamestate.__init__(self)
		
		self.resources = resources
		self.panel = resources.panel
		
		# Which player won
		self.playerWon = False
		
		# Register default eventhandler self
		self.eventhandler = self
		
	
	# Override
	def doTime(self, net):
		
		Gamestate.doTime(self, net)
		##############
		## Draw
		##############
		glClear(GL_COLOR_BUFFER_BIT)
		self.render()
		glFlush()
		pygame.display.flip()
		
	
	# Override
	def setOwnPlayer(self, player):
		Gamestate.setOwnPlayer(self, player)
		# Also register to panel
		self.panel.ownplayer = player
		
		
	def setEventHandler(self, eh):
		self.eventhandler = eh
		
	
	# Override
	def loadMapFile(self, filename, resources):
		Gamestate.loadMapFile(self, filename, resources)
		# Also pre-render background
		render.prerender(self, resources)
		
	def render(self):
	
		#glEnable(GL_TEXTURE_2D)
		#glEnable(GL_BLEND)
		#glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
		
		# Railway track & coal bases
		self.resources.renderBg()
		
		# Switch arrows
		for sw in self.switches:
			sw.render()
		
		# Trains, steam and scores
		for pl in self.players:
			pl.render()
		
		for pl in self.players:
			pl.train.renderSteam()
			
		self.renderScores()			
		
		# Panel
		self.panel.render()
		

	# Render scores and Action Points
	def renderScores(self):
				
		# Build a score list
		lst = []
		for pl in self.players:
			# Also check for win condition
			if not self.playerWon:
				if self.maxpoints < 10:
					# Carriage number race
					if pl.train.cars == self.maxpoints:
						# Player won!
						self.playerWon = pl
				else:
					# Score race
					if pl.score > self.maxpoints:
						# Player won!
						self.playerWon = pl
	
			lst.append( (int(pl.score), pl.color, pl.name) )
		
		def sort(x,y):
			if x[0] > y[0]:
				return 1
			else:
				return -1
		lst.sort(sort)
		
		i = 0
		for rec in lst:
			r,g,b = rec[1]
			self.resources.puttxt( (1000,150 + i*20), rec[2]+": "+str(rec[0]), (r, g, b, 200), right = True)
			i += 1
			
		self.resources.puttxt( (850,118), "AP: "+str(int(self.ownplayer.action_points)), (230, 230, 230, 200))
		
		if self.playerWon:
			# Render text
			sinpos = self.time/1000.0
			self.resources.puttxt( (512,420), self.playerWon.name+" won the game! Well done!", (sin(sinpos)*127+128, sin(sinpos + 2.0)*127+128, sin(sinpos + 4.0)*127+128, 240), center = True)
		
		
	# A click happened at pos, now determine, what it's target is
	def getClickEvent(self, pos):
		
		# first check for coal deposits
		for line, cb in self.coalbases.iteritems():
			if abs( vec.distance(pos, cb) ) < COALBASE_RADIUS:
				# Found a coalbase
				return (CLICK_COALBASE, line, cb)
		
		# then the own train head
		trainpos = self.ownplayer.train.getPos()
		if abs( vec.distance(pos, trainpos) ) < TRAINHEAD_RADIUS:
			# Found trainhead
			return (CLICK_TRAINHEAD, self.ownplayer.train, trainpos)
		
		# finally switches
		for sw in self.switches:
			# clear arrow highlighting for arrow under mouse
			sw.highlightArrow = False
			
			for pair, ar_pos in sw.arrows.iteritems():
				if abs( vec.distance(pos, ar_pos[0]) ) < SWITCH_RADIUS:
					# Found an arrow
					return (CLICK_SWITCH, sw, pair)
		
		# Nothing
		return False
		
		

def init():
	
	Gamestate.eventBid = gamestate_events.eventBid
	Gamestate.eventPutCoal = gamestate_events.eventPutCoal
	Gamestate.eventGetCoal = gamestate_events.eventGetCoal
	Gamestate.eventFwd = gamestate_events.eventFwd
	Gamestate.eventBack = gamestate_events.eventBack
	
	Gamestate.handleEvents = gamestate_events.handleEvents
	Gamestate.checkCollision = gamestate_collision.checkCollision
	Gamestate.loadMapFile = gamestate_fileload.loadMapFile
	Gamestate.createUpdatePackage = gamestate_syncpackage.createUpdatePackage
	Gamestate.createPersonalPackage = gamestate_syncpackage.createPersonalPackage
	Gamestate.syncToPersonalPackage = gamestate_syncpackage.syncToPersonalPackage