from constants import *
from netformats import *
from struct import *
from player import GPlayer
from string import rstrip

import socket

class Net:

	def __init__(self, gamestate, resources):
		self.resources = resources
		self.gamestate = gamestate
		gamestate.reset()
		
		self.isServer = False
			# client or server here
		self.socket = None
			# The socket used to communicate
		self.serverAddr = None
		
		self.update_pkg_count = 0
			# Used to number packages
			# Server uses to count sent
			# client to count received
		
		# Server state variables
		self.hasGameSetup = False
			# Server has set up map, etc..
		self.playerAddress = [False for i in range(0, MAXPLAYERS)]
			# Stores the players' addresses
		self.endLobby = False
			# Is lobbying finished?
		self.playerReady = [False for i in range(0, MAXPLAYERS)]
			# EVerybody ready to start?
			
	###############################
	## General disconnect function
	###############################
	# Currently used by the server mainly, to sign shutdown
	# for clients
	def disconnect(self):
		if self.isServer:
			# Broadcast to all
			msg = pack(disconFormat, NET_DISCONNECT)
			for plAddr in self.playerAddress:
				if plAddr:
					self.socket.sendto(msg, 0, plAddr)
		else:
			# Send to server
			if self.serverAddr:
				msg = pack(disconFormat, NET_DISCONNECT)
				self.socket.sendto(msg, 0, self.serverAddr)
			
	########################
	## Server functions
	########################
		
	# Server is created using this command
	def createServer(self, port = NET_DEFAULT_PORT):
		if self.socket:
			# Close current things..
			self.__init__(self.gamestate)
			pass
		
		# Register Server state
		self.isServer = True
		self.gamestate.isServer = True
		
		# Create socket
		self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		# Bind to port
		self.socket.bind( ("0.0.0.0", port) )
		self.socket.setblocking(0)
	
	
	
	# Server sets game parameters
	def setupGame(self, mapname, maxpoints):
		# Cut for package size
		mapname = mapname[:16]
		# Store locally
		self.gamestate.maxpoints = maxpoints
		# Load the map
		self.gamestate.loadMapFile(mapname, self.resources)
		
		# Place trains
		self.gamestate.placeTrains()
		
		# Register this event
		self.hasGameSetup = True
		
		# Broadcast to actual players
		msg = pack(setupGameFormat, NET_SETUP, mapname, maxpoints)
		for plAddr in self.playerAddress:
			if plAddr:
				self.socket.sendto(msg, 0, plAddr)
		
	
	
	# Server adds a player to the game
	def addPlayer(self, color, name, address):
		if not self.socket:
			raise Exception("Socket not yet inited")
		
		# If a game is already in progress, do not let connection
		if self.hasGameSetup:
			return
		
		# Check if that player already connected
		if address in self.playerAddress:
			return
		
		# Check if maxplayers reached
		pl_id = len(self.gamestate.players)
		if pl_id == MAXPLAYERS:
			# Maybe send a reject message..
			return
		
		# Store the address here, and other data in Gamestate
		self.playerAddress[pl_id] = address
		self.gamestate.addPlayer( GPlayer(pl_id, self.gamestate, name, color, self.resources) )
		
		
		# Broadcast the new player to everyone
		msg = pack(playerInitFormat, NET_PLAYER_INIT_OTHER, pl_id, color[0], color[1], color[2], name)
		i = 0
		for plAddr in self.playerAddress:
			if plAddr:
				if i == pl_id:
					# Send self init message
					self.socket.sendto(NET_PLAYER_INIT_SELF+msg[2:], 0, plAddr)
					
					## Sync player up to others
					# Send game setup, if any
					if self.hasGameSetup:
						su_msg = pack(setupGameFormat, NET_SETUP, self.gamestate.mapname, self.gamestate.maxpoints)
						self.socket.sendto(su_msg, 0, plAddr)
					
					# Send other players' data to the new player
					for pl in self.gamestate.players:
						if pl.id != pl_id:
							# Constuct packet
							other_msg = pack(playerInitFormat, NET_PLAYER_INIT_OTHER, pl.id, pl.color[0], pl.color[1], pl.color[2], pl.name)
							# Send
							self.socket.sendto(other_msg, 0, plAddr)
						
				else:
					# Send other init message
					self.socket.sendto(msg, 0, plAddr)
			# Keep track of index
			i += 1	
		
	
	# Servers sends gamestate updates to players
	def sendUpdate(self, fullSync = False):
		
		# New package
		self.update_pkg_count += 1
		# Request a public update package
		pub_update = self.gamestate.createUpdatePackage(self.update_pkg_count, fullSync)
		
		# For each player send the private update
		i = 0
		for plAddr in self.playerAddress:
			if plAddr:
				pkg = self.gamestate.createPersonalPackage(self.gamestate.players[i], pub_update)
				self.socket.sendto(pkg, 0, plAddr)
			# Keep count of index
			i += 1	
	
	###### Server inbound message handling
	def serverRead(self):
		while True:
			try:
				data, address = self.socket.recvfrom(NET_BUFFSIZE)
				
				# Get message type
				msg_type = data[:2]
				
				# Get the sender player
				try:
					pl_id = self.playerAddress.index(address)
					
					# It's an event package from a known player
					player = self.gamestate.players[pl_id]
					
					if msg_type == NET_EVENT_FWDBACK:
						dummy, fwd, press = unpack(eventFwdBackFormat, data)
						if fwd:
							self.gamestate.eventhandler.eventFwd(player, press)
						else:
							self.gamestate.eventhandler.eventBack(player, press)
						
					elif msg_type == NET_EVENT_COAL:
						dummy, put, lineid, posx, posy = unpack(eventCoalFormat, data)
						if put:
							self.gamestate.eventhandler.eventPutCoal(player)
						else:
							self.gamestate.eventhandler.eventGetCoal(player,(CLICK_COALBASE, self.gamestate.lines[lineid], (posx, posy)))
					
					elif msg_type == NET_EVENT_BID:
						dummy, state, sw_id, amount = unpack(eventBidFormat, data)
						self.gamestate.eventhandler.eventBid(player, (None, self.gamestate.switches[sw_id], 0), state, amount)
					
					elif msg_type == NET_DISCONNECT:
						# This could be handled...
						pass
						
				except ValueError:
					# Not yet in list
					# this means it must be a Connect message
					
					if msg_type == NET_PLAYER_CONNECT:
						dummy, name = unpack(playerConnectFormat, data)
						name = rstrip(name, "\0")
						print 'Connection attempt from', address
						self.addPlayer(self.gamestate.nextColor(), name, address)
					else:
						# drop
						print 'Stray package'
						return
				
			except socket.error:
				return
	
	
		
	############################
	## Client functions
	############################
	
	
	# Client send connect req using this function
	def connectToServer(self, name, address, port = NET_DEFAULT_PORT):
		if self.socket:
			# Close gamestate
			return False
		
		self.isServer = False
		self.gamestate.isServer = False
		
		try:
			self.serverAddr = (socket.gethostbyname(address), port)
		except Exception:
			return False
		self.socket= socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		self.socket.setblocking(0)
		# Create player connect message
		msg = pack(playerConnectFormat, NET_PLAYER_CONNECT, name[:16])
		# Send request
		self.socket.sendto(msg, 0, self.serverAddr)
		
		return True

		
	####### Client event handler -> send to server
	def eventFwd(self, player, press):
		msg = pack(eventFwdBackFormat, NET_EVENT_FWDBACK, True, press)
		self.socket.sendto(msg, 0, self.serverAddr)
		
	def eventBack(self, player, press):
		msg = pack(eventFwdBackFormat, NET_EVENT_FWDBACK, False, press)
		self.socket.sendto(msg, 0, self.serverAddr)
		
	def eventGetCoal(self, player, event):
		# event is (constant, line_ref of coalbase, position)
		msg = pack(eventCoalFormat, NET_EVENT_COAL, False, self.gamestate.lines.index(event[1]),  event[2][0], event[2][1])
		self.socket.sendto(msg, 0, self.serverAddr)
		
	def eventPutCoal(self, player):
		msg = pack(eventCoalFormat, NET_EVENT_COAL, True, 0, 0, 0)
		self.socket.sendto(msg, 0, self.serverAddr)
		
	def eventBid(self, player, event, state, amount):
		# event is: (constant, switch, pair)
		msg = pack(eventBidFormat, NET_EVENT_BID, state, self.gamestate.switches.index(event[1]), amount)
		self.socket.sendto(msg, 0 , self.serverAddr)
		
		
	####### Client inbound message handling
	def clientRead(self):
		while True:
			try:
				data, address = self.socket.recvfrom(NET_BUFFSIZE)
				if address != self.serverAddr:
					raise Exception("Packet from unknown source!")
					
				msg_type = data[:2]
				if msg_type == NET_UPDATE_VERSION:
					# Handle update package
					self.update_pkg_count = self.gamestate.syncToPersonalPackage(data, self.gamestate.ownplayer, self.update_pkg_count)
				
				elif msg_type == NET_SETUP:
					# Setup the game
					dummy, mapname, maxpoints = unpack(setupGameFormat, data)
					mapname = rstrip(mapname, "\0")
					print 'Setup: received map', mapname
					self.gamestate.loadMapFile(mapname, self.resources)
					self.gamestate.maxpoints = maxpoints
					
				elif msg_type == NET_PLAYER_INIT_SELF:
					# Connection accepted
					dummy, pl_id, col0, col1, col2, name = unpack(playerInitFormat, data)
					name = rstrip(name, "\0")
					print 'Connection accepted, id=', pl_id
					self.gamestate.own_id = pl_id;
					self.gamestate.addPlayerAt(pl_id, GPlayer(pl_id, self.gamestate, name, (col0, col1, col2), self.resources), LOCAL_PLAYER )
				
				elif msg_type == NET_PLAYER_INIT_OTHER:
					# Register other player 
					dummy, pl_id, col0, col1, col2, name = unpack(playerInitFormat, data)
					name = rstrip(name, "\0")
					print 'Other player, id=', pl_id, 'name is', name
					self.gamestate.addPlayerAt(pl_id, GPlayer(pl_id, self.gamestate, name, (col0, col1, col2), self.resources) )
					
				elif msg_type == NET_END_LOBBY:
					# Game is about to start from now
					# Sign ready
					pass
					
				elif msg_type == NET_DISCONNECT:
					print "Server Disconnected"
					self.gamestate.game = False
					
			except socket.error:
				return
	
	
	