from OpenGL.GL import *
from constants import *
from math import degrees, atan2, floor, ceil
import vector_op as vecop
from vector_op import add, mul
import image

import pygame
from pygame.locals import *

import config


#
# Init graphics and audio
#
def init():
	pygame.init()

	pygame.display.set_mode((config.SCREEN_W, config.SCREEN_H), OPENGL | HWSURFACE | DOUBLEBUF | FULLSCREEN*config.fullscreen)
	pygame.display.set_caption("Steamcatcher // MGS")

	glClearColor(0.0, 0.0, 0.0, 0.0)
	glClear(GL_COLOR_BUFFER_BIT)
	glMatrixMode(GL_PROJECTION)
	glOrtho(0, 1023, 0, 767, -1, 1)
	glMatrixMode(GL_MODELVIEW)
	
	glEnable(GL_TEXTURE_2D)
	glEnable(GL_BLEND)
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

#
# Convert between 1024-px system and actual resultion
#		
def convertScreenPos(pos):
	return pos[0]/float(config.SCREEN_W) * 1024.0, (config.SCREEN_H - pos[1])/float(config.SCREEN_H) * 768.0
	
	

def renderLineBase(line, image):
	
	ds = 0.0
	finished = 0.0
	
	for seg in line.segments:
		
		if ds > line.length:
			break
		
		while ds - finished < seg.length:
			pos = seg.getPos(ds - finished, 1)
			ds += RAIL_FORWARD_BASE_STEP
						
			if seg.type == 's':
				# This is a segment
				rot = degrees( atan2(seg.vector[0], -seg.vector[1]) )
			else:
				# This is an arc
				vec = vecop.vector(seg.center, pos)
				rot = degrees( atan2(vec[1], vec[0]) )
			
			image.drawCentered(pos, rot = rot)
		
		finished += seg.length

def renderLineTop(line):
	
	def calcPoses(seg, pos):
		# Calculate the normal-directed positions
		
		# Calculate normal vector
		if seg.type == 's':
			normal = (-seg.vector[1], seg.vector[0])
		else:
			normal = vecop.norm( vecop.vector(seg.center, pos) )
		
		# Points are easy to calculate from now on
		poses = ( add(mul(RAIL_INNER_SIDE, normal), pos), add(mul(RAIL_OUTER_SIDE, normal), pos), add(mul(-RAIL_INNER_SIDE, normal), pos), add(mul(-RAIL_OUTER_SIDE, normal), pos) )
		
		return poses

	line_prev = 0.0	
		
	glLoadIdentity()
	for seg in line.segments:
	
		ds = 0.0	
		prev_poses = calcPoses(seg, seg.getPos(0.0, 1))
		
		while ds < seg.length:
			
			ds += RAIL_FORWARD_STEP
			# Clip to range
			ds = min(seg.length, ds)
			
			color = (150, 150, 170)
			# Draw coal-line darker
			if line.coal:
				line_s = line_prev + ds
				if (line.coal[0] <= line_s) and (line_s <= line.coal[1]):
					color = (80, 80, 100)
			
			poses = calcPoses(seg, seg.getPos(ds, 1))
			
			# Render here
			glColor3ubv(color)
			glBegin(GL_QUADS)
			#print 'X', poses[0], poses[1], prev_poses[1], prev_poses[0]
			glVertex2fv(poses[0])
			glVertex2fv(poses[1])
			glVertex2fv(prev_poses[1])
			glVertex2fv(prev_poses[0])
			glEnd()
			
			glBegin(GL_QUADS)
			glVertex2fv(poses[3])
			glVertex2fv(poses[2])
			glVertex2fv(prev_poses[2])
			glVertex2fv(prev_poses[3])
			glEnd()
			
			prev_poses = poses
		
		line_prev += seg.length
		

#
# Pre-render tracks
#
def prerender(gamestate, resources):
	
	# determine number of tiles
	fnum_x, fnum_y = config.SCREEN_W / 128.0, config.SCREEN_H / 128.0
	num_x, num_y = floor(fnum_x), floor(fnum_y)
	
	# 1024x768-to actual ratio
	xratio = 1024.0/config.SCREEN_W
	yratio = 768.0/config.SCREEN_H
	
	render_size = (128*xratio, 128*yratio)
		
	green = image.Image("dudva.png", tex_size = (128, 128), render_size = render_size)
	coalbase = image.Image("coalbase.png")
	
	glEnable(GL_TEXTURE_2D)
	glEnable(GL_BLEND)
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
	
	
	# Green stuff
	for x in range(0, int(ceil(fnum_x)) ):
		for y in range(0, int(ceil(fnum_y)) ):
			green.draw((x*xratio*128, y*yratio*128))
	
	green.delete()
	
	# Coalbases
	for cb in gamestate.coalbases.values():
			coalbase.drawCentered(cb)
	coalbase.delete()
	
	
	# Tracks
	#rocks = image.Image("rocks.png")
	#for ln in gamestate.lines:
	#		renderLineBase(ln, rocks)
	#rocks.delete()
	
	railbase = image.Image("railbase.png")
	for ln in gamestate.lines:
			renderLineBase(ln, railbase)
	railbase.delete()
	
	glDisable(GL_TEXTURE_2D)
	for ln in gamestate.lines:
		renderLineTop(ln)
	glEnable(GL_TEXTURE_2D)
	
		
	################			
	## Finalize render
	glFlush()
				
	# save to textures
	#glPixelStorei(GL_PACK_ROW_LENGTH, 0)
  	glPixelStorei(GL_PACK_ALIGNMENT, 1)
	
	# If a right-aligned tile is required
	x_righttile = num_x != fnum_x
	
	#y_bottomtile = num_y != floor(num_y)
	# bottom line is not required, since the panel will cover it
	
	for y in range(int(num_y), -1, -1):
		for x in range(0, int(num_x)):
			
			stringdata = glReadPixels( x*128, y*128, 128, 128, GL_RGB, GL_UNSIGNED_BYTE)
			
			pos = (128*x*xratio, 128*y*yratio)
			resources.bg.append( (image.Image(strdata = stringdata, filter = GL_NEAREST, tex_size = (128, 128), render_size = render_size), pos) )
						
		if x_righttile:
			# Also take a right-aligned shot
			
			
			stringdata = glReadPixels( config.SCREEN_W-128, y*128, 128, 128, GL_RGB, GL_UNSIGNED_BYTE)
			
			resources.bg.append( (image.Image(strdata = stringdata, filter = GL_NEAREST,  tex_size = (128, 128), render_size = render_size), ((config.SCREEN_W-128)*xratio, yratio*y*128)) )
			
	
