import sets

import bid
import vector_op as vec
from math import atan2, degrees
from constants import *


#
# Ends of lines connect to the switch.
# A switch is able to connect two lines
#
class Switch:

	def __init__(self, id, gamestate):
		self.gamestate = gamestate
		self.id = id
		self.lines = sets.Set()
			# To cache which lines connect here
			# this is for near-train checks
		self.states = []
			# contains ((line_ref1, 1|-1),(line_ref2, 1|-1)) pairs
			# that represent the connected line-ends
		self.state = 0
			# The actual state
		self.bid = bid.Bid(gamestate, self)
			# The bid object of this switch
		self.trainPassing = False
			# Stores if a train is passing
		self.trainStillPassing = False
			# Auxillary member
		
		self.changed = True
			# for network resend - signs if needs to be updated
			
	
	
	# These founctions are called before and after
	# moving the trains, respectively
	# The moving train which is passing, will set
	# the trainStillPassing to True, if it really is
	def beginRound(self):
		self.trainStillPassing = False
	
	def endRound(self):
		if self.trainPassing != self.trainStillPassing:
			self.changed = True
		
		self.trainPassing = self.trainStillPassing
			
		
		
	
	# Inserts a ((lineref, end),(lineref, end)) state
	def addState(self, pairs):
		self.states.append(pairs)
		# Register switch reference in lines
		for pair in pairs:
			pair[SW_LINE_REF_IDX].setSwitch(self, pair[SW_LINE_END_IDX])
			# Also put into the lines set
			self.lines.add( pair[SW_LINE_REF_IDX] )

	
	# If the switch has a pair with memeber '(line, end)',
	# and this pair is the current state, returns the
	# other member
	# If that pair is closed, returns False
	#
	# trainEnd signs how to set the trainStillPassing flag
	# False means do not change
	# True means set True
	def getNextLine(self, pair, trainEnd):
		### DEBUG
		if not self.bid.frozen:
			raise Exception("Switch is not yet frozen!")
		
		if trainEnd:
			self.trainStillPassing = True	
		
		# Check if switch is in right state
		for i in (0,1):
			if self.states[self.state][i] == pair:
				return self.states[self.state][1-i]
		
		# No luck here
		return False


	# If the switch has any pairs with a member '(line, end)'
	# then returns the other members
	# else it returns []
	def getPairedLines(self, pair, returnStateIdx = False):
		
		result = []
		
		j = 0
		for st in self.states:
			for i in (0,1):
				if st[i] == pair:
					if returnStateIdx:
						result.append( (st[1-i], j)  ) 
					else:
						result.append(st[1-i])
			j += 1
		
		return result
	


	# Returns if any train is near the switch
	def isTrainNear(self):
		# Check all the trains, wether their
		# head or tail has a common line with the switch
		for pl in self.gamestate.players:
			head = pl.train.head
			tail = pl.train.tail
			
			for line, pos, end in (head, tail):
				if (line in self.lines) and ( line.distance(pos, end, self) <= SWITCH_RANGE ):
					return True
		
		# No trains found
		return False

	# Refreshes the state of the switch
	# This mainly includes freeze state
	def refresh(self):
		trainNear = self.isTrainNear()
		#print "trainNear:", trainNear, self.id
		#print "self.trainPassing", self.trainPassing, self.id
		
		if self.bid.frozen:
			if not self.trainPassing and not trainNear:
				# unfreeze bid
				self.bid.unfreeze()
				self.changed = True
				#print 'Unfrozen SW', self.id
		else:
			# Not frozen now
			if trainNear:
				# let's freeze
				#print 'Frozen to', self.bid.topstate, 'SW', self.id
				self.state = self.bid.topstate;
				self.bid.freeze()
				self.changed = True
	
	
	
	# The player referenced by 'player' raises the
	# bid and wants 'state'
	# If state is False, then she will re-raise
	# her last state
	def bidForState(self, player, state, amount):
		self.bid.bidForState(player, state, amount)
		
		
		
		

#
# Graphical Switch, with arrows
#

class GSwitch(Switch):
	def __init__(self, id, gamestate, resources):
		Switch.__init__(self, id, gamestate)
		
		self.resources = resources
		self.arrow_img = resources.arrow
			# Image
		self.arrows = {}
			# Contains (line, end) - (position, rotation)  pairs
		
		self.pos = False
			# This will be filled in by addState
			
		self.highlightArrow = False
			# False, or an arrow to highlight (and it's paired arrows, too)
			
		## Store own player's state preference for this switch
		self.preferredState = False
			# False, or the index of the state
		self.firstChosen = False
			# In a multi-arrow select, wether the first arrow (pair) is chosen
			# False if not, else the pair
			
	
	def clickArrow(self, pair):
		# If already chosen first, then either this is a pair of it,
		# and thus a state is selected, ot not a pair, then all selection
		# state is rendered void
		#print 'ClickArrow'
		if self.firstChosen:
			#print 'first exists'
			try:
				idx = self.states.index( (pair, self.firstChosen) )
			except ValueError:
				# not found
				try:
					idx = self.states.index( (self.firstChosen, pair) )
				except ValueError:
					# absolutely not found :)
					self.firstChosen = False
					#print 'pair not found'
					return
			
			#print 'pair found'
			# Here we have the state id
			self.firstChosen = False
			self.preferredState = idx
		else:
			# Get the pairs of that line. If there's only one, we've got the state
			# Else let this be the first chosen
			paired = self.getPairedLines(pair, True)
			if len(paired) == 1:
				#print 'found state: ', paired[0][1]
				self.firstChosen = False
				self.preferredState = paired[0][1]
			else:
				#print 'chosen as first'
				self.firstChosen = pair
				self.preferredState = False
				
				
			
	
	# Override to store arrows
	def addState(self, pairs):
		
		if not self.pos:
			pair = pairs[0]
			pos = pair[0].getPos(0.01, pair[1])
			# Move it center-ways
			self.pos = vec.add(pos, vec.mul(BID_TEXT_DIST, vec.norm(vec.vector(pos, (512, 384)))))
			
	
		# Adds new line-end pairs to the arrow list
		for pair in pairs:
			ps = self.getPairedLines(pair)
			if len(ps) == 0:
				# New one, calculate position and rotation
				pos1 = pair[0].getPos(SWITCH_ARROW_DIST1, pair[1])
				pos2 = pair[0].getPos(SWITCH_ARROW_DIST2, pair[1])
				
				center = vec.mul(0.5, vec.add(pos1, pos2))
				# Get rotation
				v = vec.vector(pos1, pos2)
				rotate = degrees( atan2(v[1], v[0]) )
				# Add
				self.arrows[pair] = (center, rotate)
		
		# Call normal behaviour
		Switch.addState(self, pairs)
	
	def render(self):
		# Draw all arrows, and draw the top-state arrow
		# With the color of the current top-bidder player
		for ar in self.arrows.values():
			self.arrow_img.drawCentered(ar[0], ARROW_COLOR, rot = ar[1])
		
		# Highlight either topplayers's pair with his color
		# or the actual state with a neutral color
		if self.bid.topplayer:
			stat, col = self.bid.topstate, self.bid.topplayer.color
		else:
			stat = self.state
			col = NEUTRAL_ARROW_COLOR
		
		# Frozen color is a bit darker
		if self.bid.frozen:
			col = (max(0,col[0]-40), max(0,col[1]-40), max(0,col[2]-40))

				
		for pair in self.states[stat]:
			pos, rot = self.arrows[pair]
			self.arrow_img.drawCentered(pos, col, True, rot)
			
		# Highlight under mouse:
		if self.highlightArrow:
			all = self.getPairedLines(self.highlightArrow)
			all.append(self.highlightArrow)
				# EVeryone to highlite is in the list now
			
			for arrow in all:
				arrow_pos = self.arrows[arrow]
				self.arrow_img.drawCentered(arrow_pos[0], ARROW_HIGHLIGHT_COLOR, rot = arrow_pos[1])
				
		# If first is chosen, also highligt that
		if self.firstChosen:
			pos, rot = self.arrows[self.firstChosen]
			self.arrow_img.drawCentered(pos, self.gamestate.ownplayer.color, True, rot)	
		
		# Draw the total amount/ amount we have to pay
		# to take over the switch
		
		if not self.bid.frozen:
			if self.bid.topplayer:
				#print 'delta_bid, topbid, own_bid:', self.bid.delta_bid, self.bid.topbid, self.bid.player_bids[self.gamestate.own_id]
				amount = self.bid.delta_bid + self.bid.topbid - self.bid.player_bids[self.gamestate.own_id]
				text = str(amount)	#+"/"+str(self.bid.topbid)
				r,g,b = self.bid.topplayer.color
				self.resources.puttxt(self.pos, text, (r,g,b,250), True)
