from line import getLinePos, invertedLinePos, get2DLinePos

from constants import *

class Train:

	def __init__(self, gamestate, player):
		self.gamestate = gamestate
		self.player = player
		
		self.head = (None, 0, 1)
			# (line_ref, scalar pos, 1|-1)
		self.tail = (None, 0, 1)
			# If the number of cars (without the engine) is C,
			# then the head-tail distance is always
			# (C+1) * CAR_LENGTH
			# 
			# The direction of the head and the tail is
			# globally opposite. For example, when
			# calculating tail pos from head pos, tail
			# pos is the inverted head direction + C.
			# Similiarly, head pos is the inverted tail
			# direction + C
		self.cars = DEFAULT_CARS	# number of cars pulled
		
		self.coal = int(MAX_COAL / 2)
		
		self.movesForward = True
		self.v = 0;
			# speed. It's always greater then zero. If it's below zero, 
			# then we clip it, and assume, that the train has stopped
			# Note: moving direction is signed by movesForward
		self.maxforce = 0;
			# MAximal force that can be set. This changes accordingly
			# to coal input, between 0 and MAX_FORCE
		self.force = 0
			# Can be set between BR_FORCE and MAX_FORCE
			# The actual force is force - friction force
		self.isBraking = False
			# signs if the train is in a complete stop
			# being in this state, the next
			# forward/backward command will determine it's
			# moving direction
		self.doSpeedup = False
		self.doBrake = False
			# If set, move() calls the appropriate function

		
	# Returns train length	
	def getLength(self):
		return (self.cars+1)*CAR_LENGTH
	
	# Resets a train to default standing state
	def defaultPos(self):
		self.movesForward = True
		self.v = 0;
		self.maxforce = 0;
		self.force = 0
		self.isBraking = False
		self.doSpeedup = False
		self.doBrake = False
		
			
	# The forward button was pressed
	# If the train is going forward, this will
	# increase the force (up to max force)
	#
	# If going backward, this will take back
	# from the force (clipped to BR_FORCE). If the 
	# force is already zero
	def speedup(self, dt):
		if self.movesForward:
			self.force += DELTA_FORCE
		else:
			# Going backward makes a slower speedup
			self.force += DELTA_FORCE * BACKWARD_FORCE_RATIO
			
		self.force = min(self.maxforce, self.force)
		#print 'speedup force=', self.force
				
	def brake(self, dt):
		self.force -= DELTA_FORCE
		self.force = max(-self.maxforce, self.force)
		#print 'brake force=', self.force
	
	# Press is True, release is False
	def forward(self, press):
		if self.isBraking:
			# Not braking anymore, go forward
			self.movesForward = True
			self.isBraking = False
			
		if self.movesForward:
			self.doSpeedup = press
		else:
			self.doBrake = press
		
	
	def backward(self, press):
		if self.isBraking:
			# Not braking anymore, go backward
			self.movesForward = False
			self.isBraking = False
	
		if self.movesForward:
			self.doBrake = press
		else:
			self.doSpeedup = press
		
	
	# Puts a coal in the engine chamber.
	# This will increase the maximum force, if it can be?
	def putCoal(self):
		if self.coal > 0:
			# Remove one coal
			self.coal -= 1
			# Add boost
			self.maxforce += COAL_FORCE_BOOST
			# Clip
			self.maxforce = min(self.maxforce, MAX_FORCE)
			#print 'coal to the fire! coal=', self.coal, 'maxF=', self.maxforce
			#print 'speed', self.v, 'fwd', self.movesForward, 'force=', self.force
	
	
	def addCoal(self, num):
		self.coal += num
		if self.coal > MAX_COAL:
			self.coal = MAX_COAL
		
		
	def addCars(self, num):
		if self.cars == MAX_CARS:
			return
		
		self.cars += num
		self.tail = (self.tail[0], self.tail[1] + CAR_LENGTH*num, self.tail[2])
		
	
	def removeCar(self):
		if self.cars == 0:
			return
			
		self.cars -= 1
		self.tail = (self.tail[0], self.tail[1] - CAR_LENGTH, self.tail[2])
		
	def move(self, dt):
		if self.doSpeedup:	
			#print 'doSpeedup'
			self.speedup(dt)
		elif self.doBrake:
			#print 'doBrake'
			self.brake(dt)
	
		# update max force and force accordingly
		self.maxforce = max (0.0, self.maxforce - DELTA_MAXFORCE * dt)
		
		if self.movesForward:
			mf = self.maxforce
		else:
			# Going backward makes a harder maxforce limit
			mf = self.maxforce * BACKWARD_MAXFORCE_RATIO
			
		self.force = min(self.force, mf)
		
		# update actual force state
		f = self.force - FRICTION_FORCE * self.v
		
		# recalc speed using force and number of cars
		self.v += f / (1 + self.cars)*2.0 * dt
		# if it's negative, clip to zero, and set
		# force to zero, and set Braking to toggle
		if self.v < 0:
			#print 'Brake=', self.doBrake, 'Speedup=', self.doSpeedup
			#print 'V < 0, V = ', self.v, 'Forward=', self.movesForward
			self.v = abs( self.v )
			# self.isBraking = True
			self.force *= -1.0
			self.movesForward = not self.movesForward
			#print 'Changed to V = ', self.v, 'Forward=', self.movesForward
			self.doBrake, self.doSpeedup = self.doSpeedup, self.doBrake
			#print 'Brake=', self.doBrake, 'Speedup=', self.doSpeedup
			
		
		# move train on the line
		ds = self.v * dt
		
		# Which direction are we going?
		# When going backward, we have to move the tail
		# to know, if we collide into a closed switch
		
		if self.movesForward:
			ref_linepos = self.head
		else:
			ref_linepos = self.tail
			
		nextpos = getLinePos(ref_linepos, ds, True)
		
			
		if nextpos is False:
			# Bump into the end of the line
			#print 'BUMP movesFwd:', self.movesForward
			#print "old refpos:", ref_linepos
			#print "line length:", ref_linepos[0].length
			#print "train length:", self.getLength()
			
			ref_linepos = (ref_linepos[LINE_REF_IDX], ref_linepos[LINE_REF_IDX].length, ref_linepos[LINE_END_IDX])
			
			#print "refpos:", ref_linepos
			
			if self.movesForward:
				self.head = ref_linepos
				self.alignToHead()
			else:
				self.tail = ref_linepos
				###### !!! Always do aligning, when manually setting position
				self.alignToTail()
			
			# Change movement direction, 
			# and renders engine force to zero	
			self.movesForward = not self.movesForward
			self.doBrake, self.doSpeedup = self.doSpeedup, self.doBrake
			#self.force *= -1
				# leave force in peace - more natural control
			self.v *= BUMP_SPEED_RATIO
			
			
		else:
			#print 'movesFwd:', self.movesForward
			#print 'v:', self.v
			# Transit head to the next line
			if self.movesForward:
				self.head = nextpos
				self.alignToHead()
			else:
				self.tail = nextpos
				self.alignToTail()
	
	
	
	# Head position is fixed, align tail according to head
	def alignToHead(self):
		self.tail = getLinePos( invertedLinePos(self.head), (self.cars + 1)*CAR_LENGTH, True)
	
	# Tail position is fixed, align head according to tail
	def alignToTail(self):
		self.head = getLinePos( invertedLinePos(self.tail), (self.cars + 1)*CAR_LENGTH, True)
		

		
import vector_op as vec
from math import atan2, degrees
from OpenGL.GL import *
		
class GTrain(Train):
	def __init__(self, gamestate, player, resources):
		Train.__init__(self, gamestate, player)
		self.steam_img = resources.steam
		self.head_img = resources.head
		self.car_img = resources.car
		
		self.lastSteam = 0
			# When did it emit steam last (msec)
		self.steam = []
			# Stores steam particles in (position, rotation, lifetime-over) format

	
	# Returns the 2D position of the train's head's center
	def getPos(self):
		return get2DLinePos(self.head, CAR_LENGTH/2.0, True)
	
		
	# Override
	def move(self, dt):
		Train.move(self, dt)
		# Keep account of steam
		# Generate new particle every MAX_FORCE/max( abs(force), 1)/K secons
		time = self.gamestate.time
		if self.lastSteam + 100.0 * MAX_FORCE / max(abs(self.force), 1.0) <= time:
			self.lastSteam = time
			# Append to the end of the list
			self.steam.append((self.getPos(), pow(time*0.792359,2), self.lastSteam + STEAM_LIFE_TIME))
			
		# Remove old steam partices
		while len(self.steam) and (self.steam[0][2] <= time):
			self.steam.pop(0)
		
	# Render the cars
	def render(self):
				
		for i in range(0, self.cars + 1):
			# What to draw
			what = self.car_img
			if i == 0:
				what = self.head_img
			
			pos1 = get2DLinePos(self.head, i*CAR_LENGTH, True)
			pos2 = get2DLinePos(self.head, (i+1)*CAR_LENGTH, True)
			
			# Now draw the cars between the two positions as vector
			center = vec.mul(0.5, vec.add(pos1, pos2))
			# Get rotation
			v = vec.vector(pos2, pos1)
			rotate = degrees( atan2(v[1], v[0]) ) - 90.0
			# Draw
			what[0].drawCentered(center, self.player.color, True, rotate)
			what[1].drawCentered(center, rot = rotate)
		
	# Render steam in last-pass, separately
	def renderSteam(self):
		for steam in self.steam:
			# Linear fade out
			alpha = (steam[2] - self.gamestate.time) / float(STEAM_LIFE_TIME)*2.0
			scale = 1.0 - 0.3*alpha
			self.steam_img.drawCentered(steam[0], rot=steam[1], alpha=alpha, scale=scale)